/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "Options.h"
#include "IntelAI.h"
#include <map>

class CBotf2Doc;
class CSectorAI;

using namespace std;
/**
 * Diese Klasse beinhaltet zustzliche Prioritten fr den Schiffsbau und Truppenbau, sowie den Geheimdienst.
 * Diese Prioritten werden dann bei der Ausfhrung der System-KI mitbeachtet.
 */
class CAIPrios
{
public:
	/// Konstruktor
	CAIPrios(CBotf2Doc* pDoc);

	/// Destruktor
	~CAIPrios(void);

	// Zugriffsfunktionen
	/// Funtkion gibt die Prioritten ein Kolonieschiff zu bauen zurck.
	BYTE GetColoShipPrio(const CString& sRaceID) {return m_mColoShipPrio[sRaceID];}

	/// Funtkion gibt die Prioritten ein Kolonieschiff zu bauen zurck.
	BYTE GetTransportShipPrio(const CString& sRaceID) {return m_mTransportPrio[sRaceID];}

	/// Funtkion gibt die Prioritten ein Kriegsschiff zu bauen zurck.
	BYTE GetCombatShipPrio(const CString& sRaceID) {return m_mCombatShipPrio[sRaceID];}

	/// Funktion gibt einen Zeiger auf das Geheimdienstobjekt fr eine bestimmte Rasse zurck.
	/// @param race Rasse
	CIntelAI* GetIntelAI(void) {return &m_IntelAI;}

	// sonstige Funktionen
	/// Funktion sollte aufgerufen werden, wenn diese Prioritt gewhlt wurde. Denn dann verringert sich
	/// diese Prioritt.
	void ChoosedColoShipPrio(const CString& sRaceID) {m_mColoShipPrio[sRaceID] /= 2;}

	/// Funktion sollte aufgerufen werden, wenn diese Prioritt gewhlt wurde. Denn dann verringert sich
	/// diese Prioritt.
	void ChoosedTransportShipPrio(const CString& sRaceID) {m_mTransportPrio[sRaceID] /= 2;}

	/// Funktion sollte aufgerufen werden, wenn diese Prioritt gewhlt wurde. Denn dann verringert sich
	/// diese Prioritt.
	void ChoosedCombatShipPrio(const CString& sRaceID) {m_mCombatShipPrio[sRaceID] /= 2;}

	/// Diese Funktion berechnet die Prioritten der einzelnen Majorrassen, wann sie ein Kolonieschiff in Auftrag
	/// geben sollen.
	void CalcShipPrios(CSectorAI* sectorAI);

	/// Funktion lscht alle vorher berechneten Prioritten.
	void Clear(void);

private:
	// Attribute
	/// Beinhaltet die Prioritt einer Rasse ein Kolonieschiff zu bauen.
	map<CString, BYTE> m_mColoShipPrio;

	/// Beinhaltet die Prioritt Truppentransporter zu bauen.
	map<CString, BYTE> m_mTransportPrio;

	/// Beinhaltet die Prioritt Kriegschiffe zu bauen.
	map<CString, BYTE> m_mCombatShipPrio;

	/// Beinhaltet Prioritten fr den Geheimdienst und stellt Funktionen zur Steuerung der KI bereit
	CIntelAI m_IntelAI;

	/// Ein Zeiger auf das Document.
	CBotf2Doc* m_pDoc;
};

/*
Schiffsangriff:

Wann wird ein Zielsektor ermittelt?
-----------------------------------
Genau dann, wenn wir angegriffen werden, einen Angriff vorbereiten wollen (noch kein Krieg) oder im Krieg mit einer Rasse
sind und sofort angreifen wollen.


Wie wird ein Zielsektor ermittelt?
----------------------------------
Wir mssen zwischen Offensive und Defensive unterscheiden. Wollen wir angreifen haben wir genug Zeit die Schiffe zu
sammeln. Mssen wir uns verteidigen, so mssen wir auch sammeln, sollten aber auch angreifen, wenn unsere Flotte
insgesamt schwcher ist. Einen Zielsektor in der Offensive sollte eine gegnerische Flotte oder ein gegnerischer
Aussenposten oder auch ein gegnerisches System sein.


Wie wird die Flotte gesammelt?
------------------------------
Zuerst wird der Sektor gesucht, in dem wir die meisten/strksten Schiffe stationiert haben. All diese Schiffe werden
zu einer Flotte zusammengefasst und fliegen zum Zielsektor. Doch sie fliegen noch nicht direkt zum Zielsektor, sondern
halten rand()%3+1 Felder davor an. Um dies rauszubekommen werden einfach die letzten rand()%3+1 Flugpfadeintrge der
Flotte entfernt. Dann wird gewartet, bis die anderen Schiffsflotten auf diesem Feld eintreffen. Ich schlage mal vor,
dass ca. 80% - 90% der Flotte auf diesem Feld eingetroffen sein muss oder die Schiffsstrke in diesem Feld schon strker
als die Schiffsstrke im Zielsektor ist. Trifft eine der beiden Bedingungen ein wird der Kurs zum Zielsektor gesetzt.

*/