/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "Options.h"
#include <map>

using namespace std;

class CBotf2Doc;
class CSectorAI;
class CShipAI
{
public:
	/// Konstruktor
	CShipAI(CBotf2Doc* pDoc);
	
	/// Destruktor
	~CShipAI(void);

	/// Diese Funktion erteilt allen Schiffen aller computergesteuerten Rassen Befehle.
	void CalculateShipOrders(CSectorAI* SectorAI);

private:
	/// Funktion erteilt einen Terraformbefehl, sofern dies auch mglich ist. bergebn wird dafr der index
	/// <code>index</code> des Schiffes aus dem Schiffsarray des Documents. Der Rckgabewert der Funktion ist
	/// <code>TRUE</code>, wenn ein Terraformbefehl gegeben werden knnte.
	BOOLEAN DoTerraform(int index);

	/// Funktion erteilt einen Kolonisierungsbefehl, sofern dies auch mglich ist. bergeben wird dafr der index
	/// <code>index</code> des Schiffes aus dem Schiffsarray des Dokuments. Der Rckgabewert der Funktion ist
	/// <code>TRUE</code>, wenn ein Kolonisierungsbefehl gegeben werden knnte.
	BOOLEAN DoColonize(int index);

	/// Funktion schickt Kriegsschiffe zu einem mglichen Offensivziel. Unter Umstnden wird auch der Befehl zum
	/// Angriff automatisch erteilt. Auch kann es passieren, das diese Funktion die Kriegsschiffe zu einem
	/// gemeinsamen Sammelpunkt schickt. bergebn wird dafr der index <code>index</code> des Schiffes aus dem
	/// Schiffsarray des Documents. Der Rckgabewert der Funktion ist <code>TRUE</code>, wenn ein Bewegungsbefehl
	/// gegeben werden knnte.
	BOOLEAN DoAttackMove(int index);

	/// Funktion schickt Kriegsschiffe zu einem mglichen System, welches Bombardiert werden knnte. bergeben wird
	/// dafr der index <code>index</code> des Schiffes aus dem Schiffsarray des Documents. Der Rckgabewert der
	/// Funktion ist <code>TRUE</code>, wenn ein Bewegungsbefehl gegeben werden konnte oder eine Bombardierung
	/// befohlen wurde oder das Schiff auf Verstrkung zur Bombardierung im System wartet.
	BOOLEAN DoBombardSystem(int index);

	/// Funktion erteilt einen Tarnbefehl, wenn <code>camouflage</code> wahr ist. Ist <code>camouflage</code> falsch,
	/// wird ein Enttarnbefehl gegeben. bergebn wird dafr der index <code>index</code> des Schiffes aus dem
	/// Schiffsarray des Documents. Der Rckgabewert der Funktion ist <code>TRUE</code>, wenn der Befehl gegeben wurde.
	BOOLEAN DoCamouflage(int index, BOOLEAN camouflage = TRUE);

	/// Funktion erteilt einen Auenpostenbaubefehl, sofern dies auch mglich ist. bergeben wird dafr der index
	/// <code>index</code> des Schiffes aus dem Schiffsarray des Dokuments. Der Rckgabewert der Funktion ist
	/// <code>TRUE</code>, wenn ein Auenpostenbaubefehl gegeben werden knnte.
	BOOLEAN DoStationBuild(int index);

	/// Funktion erstellt eine Flotte. Schiffe werden der Flotte nur hinzugefgt, wenn diese bestimmte Voraussetzungen erfllen.
	/// So muss der ungefhre Schiffstyp bereinstimmen (Combat <-> NonCombat) sowie die Geschwindigkeit des Schiffes.
	/// @param index Index des aktuellen Schiffes im Array.
	void DoMakeFleet(int index);
	
	/// Funkion berechnet einen mglichen Angriffssektor, welcher spter gesammelt angegriffen werden kann.
	void CalcAttackSector(void);

	/// Funktion berechnet einen mglich besten Sektor fr eine Bombardierung. Wurde solch ein Sektor ermittelt hat dieser
	/// die allerhchste Prioritt.
	void CalcBombardSector(void);

	/// Ein Zeiger auf das Document.
	CBotf2Doc* m_pDoc;

	/// Ein Zeiger auf das SectorAI-Objekt
	CSectorAI* m_pSectorAI;

	map<CString, CPoint> m_AttackSector;	///< der globale Angriffssektor der einzelnen Rassen

	map<CString, CPoint> m_BombardSector;	///< der globale Bombardierungssektor der einzelnen Rassen

	map<CString, int> m_iAverageMoral;		///< die durchschnittliche Moral in allen Systemen der einzelnen Rassen
};
