/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once

// forward declaration
class CBotf2Doc;
class CMajor;
class CSystemAI
{
public:
	/// Konstruktor
	CSystemAI(CBotf2Doc* pDoc);

	/// Destruktor
	~CSystemAI(void);

	/// Diese Funktion fhrt die Berechnungen fr die knstliche Intelligenz in einem System aus und macht
	/// alle mglichen Eintrge und Vernderungen. Als Paramter wird dafr die Koordinate <code>ko</code>
	/// des Systems bergeben.
	void ExecuteSystemAI(CPoint ko);

private:
	/// Diese Funktion kauft unter Umstnden den aktuellen Bauauftrag. Somit kommt der Ausbaue schneller voran.
	void PerhapsBuy();

	/// Diese Funktion legt die Prioritten an, mit welcher ein bestimmtes Arbeitergebude gebaut werden soll.
	void CalcPriorities();

	/// Diese Funktion whlt ein zu bauendes Gebude aus der Liste der baubaren Gebude. Es werden nur Gebude
	/// ausgewhlt, welche in die Priorittenliste passen. Der Rckgabewert ist die ID des Bauauftrages. 
	/// Wird <code>0</code> zurckgegeben, so wurde kein Gebude gefunden, welches in das Anforderungsprofil passt.
	int ChooseBuilding();

	/// Diese Funktion whlt ein zu bauendes Schiff aus der Liste der baubaren Schiffe. Der Rckgabewert ist die ID
	/// des Schiffes (also grer 10000!). Wird <code>0</code> zurckgegeben, so wurde kein baubares Schiffe gefunden.
	/// bergeben wird dafr die Prioritt <code>prio</code> des zu bauenden Schiffes (mittels Funktion <function>
	/// GetShipBuildPrios</function> ermitteln) und Wahrheitswertr, in denen steht welcher Schiffstyp gebaut werden soll.
	int ChooseShip(int prio, BOOLEAN chooseCombatship, BOOLEAN chooseColoship, BOOLEAN chooseTransport);
	
	/// Funktion berprft ob das Gebude mit der ID <code>id</code> in die Bauliste gesetzt werden kann.
	/// Wenn es in die Bauliste gesetzt werden konnte gibt die Funktion ein <code>TRUE</code> zurck.
	BOOLEAN MakeEntryInAssemblyList(short id);

	/// Diese Funktion besetzt die Gebude mit Arbeitern.
	void AssignWorkers();

	/// Diese Funktion versucht Arbeiter aus Bereichen abzuziehen. Dabei werden aber keine Arbeiter aus dem Nahrungs-
	/// und Energiebereich abgezogen. Die Funktion berechnet die neue Arbeiterverteilung, auch die freien Arbeiter, und
	/// gibt ein <code>TRUE</code> zurck, wenn Arbeiter abgezogen werden konnten. Ansonsten gibt sie <code>FALSE</code>
	/// zurck.
	BOOLEAN DumpWorker();

	/// Funktion reit berflssige Nahrungs- und Energiegebude im System ab.
	void ScrapBuildings();

	/// Diese Funktion berechnet die neue Nahrungs-, Industrie- und Energieproduktion im System. Sie sollte aufgerufen
	/// werden, wenn Arbeiter aus diesen Gebieten verndert wurden. Dabei werden aber alle anderen Produktionen
	/// gelscht.
	void CalcProd();

	/// Diese Funktion legt mglicherweise eine Handelsroute zu einem anderen System an. Dadurch wird halt auch die
	/// Beziehung zu Minorraces verbessert.
	void ApplyTradeRoutes();

	/// Diese Funktion berechnet die Prioritt, mit welcher ein gewisser Schiffstyp gebaut werden soll. bergeben werden
	/// dabei drei Referenzen auf Booleanvariablen, in welcher dann ein <code>TRUE</code> steht wurde ausgewhlt.
	/// @return ist die Prioritt, mit welcher der Schiffstyp gebaut werden soll.
	int GetShipBuildPrios(BOOLEAN &chooseCombatship, BOOLEAN &chooseColoship, BOOLEAN &chooseTransport);

	/// Funktion berprft, ob fr einen bestimmten Arbeitertyp auch ein Gebude in der Liste der baubaren
	/// Gebude exisitiert. Wenn dies nicht der Fall ist, so ist auch die Prioritt solch ein Gebude zu bauen
	/// gleich Null.
	/// @param whichWorker Arbeitertyp (FOOD_WORKER, INDUSTRY_WORKER usw.)
	/// @return <code>TRUE</code> wenn ein Gebude baubar ist, ansonsten <code>FALSE</code>
	BOOLEAN CheckBuilding(BYTE whichWorker) const;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
	
	/// Ein Zeiger auf das Document.
	CBotf2Doc* m_pDoc;

	/// Die Koordinate des Systems, in welchem gerade die KI-Berechnungen durchgefhrt werden sollen
	CPoint m_KO;

	/// Die Prioritten der einzelnen Arbeitergebude
	int m_iPriorities[10];

	/// Was sollte am ehesten gebaut werden? 0 - Gebude, 1 - Upgrade, 2 - Schiff, 3 - Truppen
	BYTE m_bBuildWhat;

	/// Wurden die Prioritten berechnet, so nimmt diese Variable den Wert <code>TRUE</code> an. Ansonsten ist
	/// ihr Wert <code>FALSE</code>.
	BOOLEAN m_bCalcedPrio;

	/// auf diese Variable wird whrend der Berechnung die Majorrace gespeichert, der das System gehrt.
	CMajor* m_pMajor;
};
