/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "intelobject.h"

/**
 * Diese Klasse stellt Informationen zu einem Geheimdiensteinsatz mit irgendeinem Wirtschaftsziel bereit.
 *
 * @author Sir Pustekuchen
 * @version 0.1
 */
class CEcoIntelObj : public CIntelObject
{
	DECLARE_SERIAL (CEcoIntelObj)
public:
	/// Konstruktor
	CEcoIntelObj(void);

	/// Konstruktor mit Parameterbergabe
	/// @param sOwnerID Auslser/Eigentmer
	/// @param sEnemyID Ziel der Geheimdienstaktion
	/// @param round Runde in der die Geheimdienstaktion angelegt wurde
	/// @param isSpy handelt es sich um Spionage oder Sabotage
	/// @param ko Koordinaten des System
	/// @param id ID der betroffenen Gebude
	/// @param number Anzahl der betroffenen Gebude
	CEcoIntelObj(const CString& sOwnerID, const CString& sEnemyID, USHORT round, BOOLEAN isSpy, const CPoint &ko, USHORT id, BYTE number);

	/// Konstruktor mit Parameterbergabe
	/// @param sOwnerID Auslser/Eigentmer
	/// @param sEnemyID Ziel der Geheimdienstaktion
	/// @param round Runde in der die Geheimdienstaktion angelegt wurde
	/// @param isSpy handelt es sich um Spionage oder Sabotage
	/// @param credits Credits
	CEcoIntelObj(const CString& sOwnerID, const CString& sEnemyID, USHORT round, BOOLEAN isSpy, int credits);

	/// Destruktor
	~CEcoIntelObj(void);

	/// Kopierkonstruktor
	CEcoIntelObj(const CEcoIntelObj & rhs);

	/// Serialisierungsfunktion
	void Serialize(CArchive &ar);

	// Zugriffsfunktionen

	/// Funktion gibt die Koordinaten des Sektors zurck, auf welchen sich die Geheimdienstaktion auswirkt.
	const CPoint& GetKO() const {return m_KO;}

	/// Funktion gibt die ID des Gebudes zurck, auf welches sich die Geheimdienstaktion auswirkt.
	USHORT GetID() const {return m_nID;}

	/// Funktion gibt die Anzahl der Gebude zurck, auf welche sich die Geheimdienstaktion auswirkt.
	BYTE GetNumber() const {return m_byNumber;}

	/// Funktion gibt die Creditsmenge zurck
	int GetCredits() const {return m_iCredits;}
	
	// sonstige Funktion
	/// Funktion generiert einen Text, welcher eine Geheimdiestaktion beschreibt, fr den Auslser bzw. das Opfer
	/// dieser Aktion.
	/// @param pDoc Zeiger auf das Dokument
	/// @param n Nummer der verschiedenen Textmglichkeiten, im Normalfall <code>NULL</code>
	/// @param param Hier kann die Rasse bergeben werden, von der das Opfer denkt angegriffen worden zu sein
	void CreateText(CBotf2Doc* pDoc, BYTE n, const CString& param);

private:
	// Attribute
	CPoint m_KO;				///< Systemkoordinaten des Wirtschaftsanschlages

	BYTE m_byNumber;			///< Anzahl der spionierten/sabotierten Gebude

	USHORT m_nID;				///< ID der zu sabotierenden/sabotierten Gebude

	int m_iCredits;				///< Creditsmenge
};
