/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "afx.h"
#include "IntelAssignment.h"
#include "IntelReports.h"
#include "IntelInfo.h"
#include <map>

using namespace std;
/**
 * Diese Klasse abstrahiert den Geheimdienst einer Hauptrasse. Sie beinhaltet alle Funktionen zur Manipulation
 * von Geheimdienstwerten.
 *
 * @author Sir Pustekuchen
 * @version 0.1
 */
class CIntelligence : public CObject
{	
public:
	DECLARE_SERIAL (CIntelligence)
	
	/// Standardkonstruktor
	CIntelligence(void);

	/// Destruktor
	~CIntelligence(void);

	/// Serialisierungsfunktion
	void Serialize(CArchive &ar);

	// Zugriffsfunktionen
	/// Funktion gibt die Nummer der Rasse zurck, welche wir fr unsere Geheimdienstaten verantworlich machen wollen.
	const CString& GetResponsibleRace() const {return m_sResponsibleRace;}

	/// Funktion gibt die Anzahl der produzierten Geheimdienstpunkte zurck.
	UINT GetSecurityPoints() const {return m_iSecurityPoints;}

	/// Funktion gibt die gesammelten Punkte fr die innere Sicherheit zurck.
	UINT GetInnerSecurityStorage() const {return m_iInnerStorage;}

	/// Funktion gibt die angesammelten Punkte bei der Spionage oder der Sabotage bei einer bestimmten Rasse
	/// zurck.
	/// @param type Spionage == 0, Sabotage == 1
	/// @param sRace Rasse
	UINT GetSPStorage(BOOLEAN type, const CString& sRace) {return m_iSPStorage[type][sRace];}

	/// Funktion gibt einen Bonus auf ein Geheimdienstressort zurck.
	/// @param bonus Wirtschaft == 0, Forschung == 1, Militr == 2, Diplomatie == 3, innere Sicherheit == 4
	/// @param type Spionage == 0, Sabotage == 1
	short GetBonus(BYTE bonus, BOOLEAN type) const;

	/// Funktion gibt den Bonus der inneren Sicherheit zurck.
	short GetInnerSecurityBoni() const {return (m_nInnerSecurityBoni < -100) ? -100 : m_nInnerSecurityBoni;}

	/// Funktion gibt den Bonus fr Wirtschaft zurck.
	/// @param type Spionage == 0, Sabotage == 1
	short GetEconomyBonus(BOOLEAN type) const {return (m_nEconomyBoni[type] < -100) ? -100 : m_nEconomyBoni[type];}

	/// Funktion gibt den Bonus fr Wissenschaft zurck.
	/// @param type Spionage == 0, Sabotage == 1
	short GetScienceBonus(BOOLEAN type) const {return (m_nScienceBoni[type] < -100) ? -100 : m_nScienceBoni[type];}

	/// Funktion gibt den Bonus fr Militr zurck.
	/// @param type Spionage == 0, Sabotage == 1
	short GetMilitaryBonus(BOOLEAN type) const {return (m_nMilitaryBoni[type] < -100) ? -100 : m_nMilitaryBoni[type];}

	/// Funktion gibt die Aggressivitt bei Geheimdienstaktionen zurck.
	/// @param type Spionage == 0, Sabotage == 1
	/// @param sRace Rasse
	BYTE GetAggressiveness(BOOLEAN type, const CString& sRace) {return m_byAggressiveness[type][sRace];}

	/// Funktion gibt einen Zeiger auf die Geheimdienstreporte zurck.
	CIntelReports* GetIntelReports() {return &m_Reports;}

	/// Funktion gibt einen Zeiger auf die Geheimdienstinformationen eines Imperiums zurck.
	CIntelInfo* GetIntelInfo() {return &m_IntelInfo;}

	/// Funktion gibt einen Zeiger auf die Zuteilung der einzelnen Geheimdienstressorts zurck.
	CIntelAssignment* GetAssignment() {return &m_Assignment;}

	/// Funktion gibt einen Zeiger auf die Zuteilung der einzelnen Geheimdienstressorts zurck.
	CIntelAssignment* SetAssignment() {return &m_Assignment;}

	/// Funktion legt die Rassennummer fest.
	/// @param sRace Nummer der zugehrigen Rasse/Imperium
	void SetRaceID(const CString& sRace) {m_sRace = sRace; SetResponsibleRace(sRace);}

	/// Funktion legt die fr unsere Geheimdiensttaten verantworlich machbare Rasse fest. Somit lt sich die Sache
	/// auf eigentlich unbeteiligte Rassen abwlzen.
	/// @param responibleRace neue verantworlichmachbare Rasse
	void SetResponsibleRace(const CString& responsibleRace) {m_sResponsibleRace = responsibleRace;}

	/// Funktion setzt die Aggressivitt bei Geheimdienstaktionen fest.
	/// @param type Spionage == 0, Sabotage == 1
	/// @param sRace Rasse
	/// @param value vorsichtig == 0, normal == 1, aggressiv = 2
	void SetAggressiveness(BOOLEAN type, const CString& sRace, BYTE value) {m_byAggressiveness[type][sRace] = value;}

	/// Funktion addiert einen Wert zum vorhandenen Wert der inneren Sicherheit. Funktion berprft auch, ob der neue
	/// Wert im richtigen Bereich liegt.
	/// @param add der zu addierende Wert
	void AddInnerSecurityPoints(int add);

	/// Funktion addiert die bergebenen Punkte zum jeweiligen Lager einer Rasse. Gleichzeitig wird berprft, dass
	/// ihr neuer Wert im richtigen Bereich liegt.
	/// @param type Spionage == 0, Sabotage == 1
	/// @param sRace Rasse
	/// @param add der zu addierende Wert
	void AddSPStoragePoints(BOOLEAN type, const CString& sRace, int add);
	
	/// Funktion fgt den im Parameter bergebenen Wert dem Bonus fr die innere Sicherheit hinzu.
	/// @param add hinzuzufgender Bonus
	void AddInnerSecurityBonus(short add) {m_nInnerSecurityBoni += add;}

	/// Funktion fgt den im Parameter bergebenen Wert dem Bonus fr die Wirtschaft hinzu.
	/// @param add hinzuzufgender Bonus
	/// @param type Spionage == 0, Sabotage == 1
	void AddEconomyBonus(short add, BOOLEAN type) {m_nEconomyBoni[type] += add;}

	/// Funktion fgt den im Parameter bergebenen Wert dem Bonus fr die Wissenschaft hinzu.
	/// @param add hinzuzufgender Bonus
	/// @param type Spionage == 0, Sabotage == 1
	void AddScienceBonus(short add, BOOLEAN type) {m_nScienceBoni[type] += add;}

	/// Funktion fgt den im Parameter bergebenen Wert dem Bonus fr die Militr hinzu.
	/// @param add hinzuzufgender Bonus
	/// @param type Spionage == 0, Sabotage == 1
	void AddMilitaryBonus(short add, BOOLEAN type) {m_nMilitaryBoni[type] += add;}

	// sonstige Funktionen
	/// Funktion lscht die produzierten Geheimdienstpunkte.
	void ClearSecurityPoints() {m_iSecurityPoints = 0;}

	/// Funktion lscht alle Geheimdienstboni.
	void ClearBoni();

	/// Funktion addiert den im Paramter bergebenen Wert zu den aktuell produzierten Geheimdienstpunkten.
	/// @param add Anzahl der addierten Geheimdienstpunkte
	void AddSecurityPoints(int add);

	/// Resetfunktion fr das CIntelligence-Objekt.
	void Reset();

private:
	// Attribute
	CString m_sRace;					///< Rassen-ID des Imperiums zu welchem dieses Geheimdienstobjekt gehrt

	CString m_sResponsibleRace;			///< Rasse welche fr unsere Geheimdiensttaten verantwortlich gemacht werden kann

	UINT m_iSecurityPoints;				///< globale Anzahl der produzierten Geheimdienstpunkte

	UINT m_iInnerStorage;				///< Lager der angesammelten Geheimdienstpunkte fr die innere Sicherheit

	map<CString, UINT> m_iSPStorage[2];	///< Lager der angesammelten Spionage- und Sabotagepunkte bei einer bestimmten Rasse

	short m_nInnerSecurityBoni;			///< Bonus auf die innere Sicherheit

	short m_nEconomyBoni[2];			///< Wirtschaftsbonus fr Spionage und Sabotage;

	short m_nScienceBoni[2];			///< Wissenschaftsbonus fr Spionage und Sabotage;

	short m_nMilitaryBoni[2];			///< Militrbonus fr Spionage und Sabotage;

	map<CString, BYTE> m_byAggressiveness[2];	///< Aggressivitt, mit der bei Spionage und Sabotage vorgegangen wird

	CIntelAssignment m_Assignment;		///< Zuteilungen auf die einzelnen Geheimdienstressorts

	CIntelReports m_Reports;			///< alle Geheimdienstreports (Aktionen)

	CIntelInfo m_IntelInfo;				///< Geheimdienstinformationen (Datensammlung ber andere Rassen)
};
