/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "intelobject.h"

/**
 * Diese Klasse stellt Informationen zu einem Geheimdiensteinsatz mit irgendeinem militrischen Ziel bereit.
 *
 * @author Sir Pustekuchen
 * @version 0.1
 */

class CMilitaryIntelObj : public CIntelObject
{
	DECLARE_SERIAL (CMilitaryIntelObj)
public:
	/// Konstruktor
	CMilitaryIntelObj(void);

	/// Konstruktor mit Parameterbergabe
	/// @param sOwnerID Auslser/Eigentmer
	/// @param sEnemyID Ziel der Geheimdienstaktion
	/// @param round Runde in der die Geheimdienstaktion angelegt wurde
	/// @param isSpy handelt es sich um Spionage oder Sabotage
	/// @param ko Koordinaten des System
	/// @param id ID der betroffenen Gebude
	CMilitaryIntelObj(const CString& sOwnerID, const CString& sEnemyID, USHORT round, BOOLEAN isSpy, const CPoint &ko, USHORT id, USHORT number,
		BOOLEAN building, BOOLEAN ship, BOOLEAN troop);

	/// Destruktor
	~CMilitaryIntelObj(void);

	/// Kopierkonstruktor
	CMilitaryIntelObj(const CMilitaryIntelObj & rhs);

	/// Serialisierungsfunktion
	void Serialize(CArchive &ar);

	// Zugriffsfunktionen
	/// Funktion gibt die Koordinaten des Sektors zurck, auf welchen sich die Geheimdienstaktion auswirkt.
	CPoint GetKO() const {return m_KO;}

	/// Funktion gibt die ID des Gebudes, des Schiffes oder der Truppe zurck, auf welches sich die Geheimdienstaktion
	/// auswirkt.
	USHORT GetID() const {return m_nID;}

	/// Funktion gibt die Anzahl der Gebude, Schiffe oder Truppen zurck, auf welche sich die Geheimdienstaktion auswirkt.
	USHORT GetNumber() const {return m_nNumber;}

	/// Funktion gibt zurck, ob das Ziel dieser Militraktion ein Gebude ist.
	BOOLEAN GetIsBuilding() const {return m_bBuilding;}

	/// Funktion gibt zurck, ob das Ziel dieser Militraktion ein Schiff/Aussenposten ist.
	BOOLEAN GetIsShip() const {return m_bShip;}

	/// Funktion gibt zurck, ob das Ziel dieser Militraktion eine Truppe ist.
	BOOLEAN GetIsTroop() const {return m_bTroop;}

	// sonstige Funktionen
	/// Funktion generiert einen Text, welcher eine Geheimdiestaktion beschreibt, fr den Auslser bzw. das Opfer
	/// dieser Aktion.
	/// @param pDoc Zeiger auf das Dokument
	/// @param n Nummer der verschiedenen Textmglichkeiten, im Normalfall <code>NULL</code>
	/// @param param Hier kann die Rasse bergeben werden, von der das Opfer denkt angegriffen worden zu sein
	void CreateText(CBotf2Doc* pDoc, BYTE n, const CString& param);

private:
	// Atribute
	CPoint m_KO;					///< Systemkoordinaten der Militrsspionage

	USHORT m_nID;					///< ID der zu sabotierenden/sabotierten Gebude, Truppen, Schiffe

	USHORT m_nNumber;				///< Anzahl der Gebude/Schiffe/Truppen

	BOOLEAN m_bTroop;				///< handelt es sich beim Ziel der Aktion um eine Truppe

	BOOLEAN m_bShip;				///< handelt es sich beim Ziel der Aktion um ein Schiff/Schiffe

	BOOLEAN m_bBuilding;			///< handelt es sich beim Ziel der Aktion um ein/mehrere Gebude
};
