/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */

#pragma once
#include <vector>

// forward declaration
class CBotf2Doc;
class CRace;
class CMajor;
class CMinor;
class CDiplomacyInfo;

/// Klasse zur Verarbeitung aller diplomatischen Ereignisse.
class CDiplomacyController
{
protected:
	/// Standardkonstruktor
	CDiplomacyController(void);

public:
	/// Standarddestruktor
	~CDiplomacyController(void);

	/// Funktion zum Versenden von diplomatischen Angeboten
	static void Send(void);

	/// Funktion zum Empfangen und Bearbeiten eines diplomatischen Angebots.
	static void Receive(void);

private:
	// private Funktionen

	/// Funktion berprft die diplomatische Konsistenz und berechnet die direkten diplomatischen Auswirkungen.
	/// Sie sollte nach <func>Receive</func> aufgerufen werden.
	/// Stellt die Funktion Probleme fest, so werden diese automatisch behoben.
	static void CalcDiplomacyFallouts(CBotf2Doc* pDoc);

	/// Funktion zum Versenden von diplomatischen Angeboten an eine Majorrace.
	/// @param pDoc Zeiger auf das Dokument
	/// @param pToMajor Zeiger auf die Empfngerrasse
	/// @param pInfo Zeiger auf aktuelles Diplomatieobjekt
	static void SendToMajor(CBotf2Doc* pDoc, CMajor* pToMajor, CDiplomacyInfo* pInfo);

	/// Funktion zum Versenden von diplomatischen Angeboten an eine Minorrace.
	/// @param pDoc Zeiger auf das Dokument
	/// @param pToMinor Zeiger auf die Empfngerrasse
	/// @param pInfo Zeiger auf aktuelles Diplomatieobjekt
	static void SendToMinor(CBotf2Doc* pDoc, CMinor* pToMinor, CDiplomacyInfo* pInfo);

	/// Funktion zum Empfangen und Bearbeiten eines diplomatischen Angebots fr eine Majorrace.
	/// @param pDoc Zeiger auf das Dokument
	/// @param pToMajor Zeiger auf die Empfngerrasse
	/// @param pInfo Zeiger auf aktuelles Diplomatieobjekt
	static void ReceiveToMajor(CBotf2Doc* pDoc, CMajor* pToMajor, CDiplomacyInfo* pInfo);
	
	/// Funktion zum Empfangen und Bearbeiten eines diplomatischen Angebots fr eine Minorrace.
	/// @param pDoc Zeiger auf das Dokument
	/// @param pToMajor Zeiger auf die Empfngerrasse
	/// @param pInfo Zeiger auf aktuelles Diplomatieobjekt
	static void ReceiveToMinor(CBotf2Doc* pDoc, CMinor* pToMinor, CDiplomacyInfo* pInfo);

	/// Funktion zum Berechnen aller betroffenen Rassen, welchen zustzlich der Krieg erklrt wird.
	/// Dies geschieht dadurch, wenn der Kriegsgegner Bndnisse oder Verteidigungspakt mit anderen Rassen besitzt.
	/// @param pDoc Zeiger auf das Dokument
	/// @param pFromMajor Zeiger auf die kriegserklrende Majorrace
	/// @param pToRace Zeiger auf die Rasse, welcher der Krieg erklrt wurde
	/// @return Feld mit allen betroffenen Rassen, denen auch der Krieg erklrt werden muss
	static std::vector<CString> GetEnemiesFromContract(CBotf2Doc* pDoc, CMajor* pFromMajor, CRace* pToRace);
	
	/// Funktion erklrt den Krieg zwischen zwei Rassen und nimmt dabei alle notwendigen Arbeiten vor.
	/// @param pFromRace Zeiger auf die kriegserklrende Rasse
	/// @param pEnemy Zeiger auf die Rasse, welcher Krieg erklrt wird
	/// @param pInfo Diplomatieobjekt
	/// @param bWithMoralEvent <code>true</code> wenn Moralevent mit eingeplant werden soll
	static void DeclareWar(CRace* pFromRace, CRace* pEnemy, CDiplomacyInfo* pInfo, bool bWithMoralEvent);
};
