/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
// Starmap.h: interface for the CStarmap class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STARMAP_H__28F3FFEA_139A_45A5_AFF3_76F41E48496B__INCLUDED_)
#define AFX_STARMAP_H__28F3FFEA_139A_45A5_AFF3_76F41E48496B__INCLUDED_

// nur zu Demozwecken, sollte auskommentiert werden!
// #define DEBUG_AI_BASE_DEMO

#include "Options.h"
#include "SortedList.h"
#include <list>
#include <set>
#include <map>

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// die verschiedenen Reichweiten; je grer der Wert, desto nher ist ein Auenposten
#define SM_RANGE_SPACE		0		///< auerhalb aller Reichweiten
#define SM_RANGE_FAR		1		///< groe Entfernung
#define SM_RANGE_MIDDLE		2		///< mittlere Entfernung
#define SM_RANGE_NEAR		3		///< kurze Entfernung

/// Makro, das prft, ob der angegebene Parameter ein gltiger Sektor ist
#define ASSERT_SECTOR_VALID(sector) \
	ASSERT(PT_IN_RECT((sector), 0, 0, STARMAP_SECTORS_HCOUNT, STARMAP_SECTORS_VCOUNT))

/// Struktur, die die Koordinaten eines Sektors aufnehmen kann
struct Sector
{
	Sector() : x(-1), y(-1) {}
	Sector(char _x, char _y) : x(_x), y(_y) {}
	Sector(const Sector &other) : x(other.x), y(other.y) {}
	int operator==(const Sector &other) const {return (other.x == x && other.y == y);}
	Sector operator=(const Sector &other) {x = other.x; y = other.y; return *this;}
	Sector operator+(const Sector &other) {return Sector(x + other.x, y + other.y);}
	int operator!=(const Sector &other) const {return (other.x != x || other.y != y);}

	char x, y;
};

/// Struktur, die Informationen eines Sektors fr die Berechnung eines krzesten Weges aufnimmt
struct PathSector
{
	bool used;			///< <code>true</code>, wenn der Knoten bereits gewhlt wurde, sonst <code>false</code>
	double distance;	///< Entfernung vom Start
	int hops;			///< Anzahl der Hops vom Start-Sektor
	Sector parent;		///< Verweis auf Elternknoten

	Sector position;	///< Koordinaten des Sektors
};

/// Komparator fr PathSector-Objekte (vergleicht die distance-Werte)
class PathSectorComparator
{
public:
	/// liefert -1 wenn a &lt; b, 0 wenn a == b und 1 wenn a &gt; b
	int operator()(const PathSector *a, const PathSector *b) const
	{
		if (a->distance < b->distance)
			return -1;
		else if (a->distance == b->distance)
			return 0;
		else
			return 1;
	}
};

/// Liste, die Nachbar-Sektoren mit Abstnden aufnehmen kann (fr CalcPath)
typedef CSortedList<PathSector, PathSectorComparator> LeafList;

/// Doppelt verkettete List mit Sector-Eintrgen
typedef std::list<Sector> SECTORLIST;

/// Nimmt Informationen ber Sektoren und deren Bewertung auf.
struct BaseSector
{
	Sector position;		///< Koordinaten des Sektors
	short points;			///< Bewertung des Sektors

	BaseSector() : position(Sector()), points(-1) {}

	bool operator<(const BaseSector &other)
	{
		// wollen absteigend sortieren
		return points > other.points;
	}

};

typedef struct RangeMap
{
	unsigned char *range;		///< Array, das die Reichweitenmatrix zeilenweise enthlt
	char w;						///< Anzahl der Spalten der Matrix
	char h;						///< Anzahl Zeilen der Matrix
	
	/// nullbasierter Index einer Spalte; die Matrix wird so ausgerichtet, dass diese Spalte ber dem Sektor eines Auenpostens steht
	char x0;
	/// nullbasierter Index einer Zeile; die Matrix wird so ausgerichtet, dass diese Zeile ber dem Sektor eines Auenpostens steht
	char y0;
}
RangeMap;

/**
 * Klasse, die fr einen Spieler smtliche Infos der Starmap enthlt. Enthlt Methoden zur Berechnung von Reichweiten,
 * Flugrouten (krzester Weg) und Sektoren zum Auenpostenbau durch die KI.
 *
 * @author CBot
 * @version 0.0.2
 */
class CSector;
class CStarmap  
{
	friend class CGalaxyMenuView;

protected:
	/// Map, die die Entfernung vom nchsten Auenposten aufnimmt; der erste Index ist die x-,
	/// der zweite die y-Koordinate eines Sektors (modelliert Einfluss, den der Auenposten hat)
	unsigned char m_Range[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];
	/// Koordinaten des aktuell ausgewhlten Sektors; (-1, -1) wenn keiner ausgewhlt ist
	Sector m_Selection;

	/// Liste der Sektoren, in denen sich ein Auenposten befindet
	SECTORLIST m_lBases;

	/// zu verwendende lokale RangeMap, fr nachfolgend hinzugefgte Auenposten
	RangeMap m_RangeMap;

	/// bestimmte Sektoren sind gefhrlich und sollten daher nicht primr angeflogen werden
	static double m_BadMapModifiers[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];

	/// Array, das Informationen zur Berechnung der krzesten Wege aufnimmt
	PathSector pathMap[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];
	/// Liste der Nachbarn, die von den aktuellen Blttern aus erreichbar sind
	/// (werden als nchstes zu Blttern des Baumes)
	LeafList leaves;

	Sector pathStart;				///< Start-Sektor beim letzten Aufruf von CalcPath(), {-1, -1} wenn nicht gesetzt
	unsigned char pathRange;		///< Reichweite beim letzten Aufruf von CalcPath(), 0 wenn nicht gesetzt


	BOOL m_bAICalculation;			///< zustzliche Berechnungen fr den automatischen Auenpostenbau aktiviert?
	char m_nAIRange;				///< maximale Reichweite, innerhalb derer Auenposten gebaut werden sollen und die zusammenhngen soll
	SECTORLIST m_lAITargets;		///< Ziele, zu denen sich bevorzugt ausgebreitet werden soll
	SECTORLIST m_lAIKnownSystems;	///< Liste von der KI bekannten Systemen (durch Scannen)

	/// Gebietszuwachs fr Sektor (x, y), wenn in diesem Sektor ein Auenposten gebaut wrde
	short m_AIRangePoints[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];

	/// fr Sektoren auerhalb der gegebenen Reichweite die Anzahl der Nachbarn innerhalb der Reichweite
	unsigned char m_AINeighbourCount[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];
	/// Bewertung fr den Zusammenhang eines Gebiets, um Lcken zu vermeiden und nicht zusammenhngende Gebiete zu verbinden
	short m_AIConnectionPoints[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];

	/// Bewertung fr bevorzugte Ausbreitungsrichtungen
	short m_AITargetPoints[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];

	/// negative Bewertung fr ein Gebiet durch feindliche Grenzen. Man sollte keinen Aussenposten auf einen freien
	/// Fleck inmitten des gegnerischen Gebiets bauen.
	short m_AIBadPoints[STARMAP_SECTORS_HCOUNT][STARMAP_SECTORS_VCOUNT];

public:
	/**
	 * @param bAICalculation aktiviert zustzliche Berechnungen, die den automatischen Auenpostenbau ermglichen
	 * @param nAIRange maximale Reichweite, innerhalb derer Auenposten gebaut werden sollen und die zusammenhngen soll
	 */
	CStarmap(BOOL bAICalculation = FALSE, char nAIRange = SM_RANGE_NEAR);
	virtual ~CStarmap();
	
	/**
	 * Funktion gibt den Wert aus der RangeMap <code>m_Range</code> an einer bestimmten Stelle <code>p</p> auf der
	 * Karte zurck.
	 */
	unsigned char GetRange(const CPoint &p) {return (3 - m_Range[p.x][p.y]);}

	/**
	 * Ermittelt die Koordinaten eines angeklickten Sektors. <code>pt</code> gibt die ungezoomten Mauskoordinaten
	 * bezglich der linken oberen Ecke der gesamten Starmap an.
	 */
	Sector GetClickedSector(const CPoint& pt) const;
	/**
	 * Liefert die Koordinaten der linken oberen Ecke des angegebenen Sektors.
	 */
	CPoint GetSectorCoords(const Sector& sector) const;

	/**
	 * Whlt den angegebenen Sektor aus. Bei Angabe von (-1, -1) hat diese Methode keine Wirkung. Zum Entfernen der
	 * Markierung muss Deselect() verwendet werden.
	 */
	void Select(const Sector &sector);
	/**
	 * Entfernt die Markierung.
	 */
	void Deselect();
	/**
	 * Liefert die Koordinaten des ausgewhlten Sektors. Liefert {-1, -1}, wenn kein Sektor ausgewhlt ist.
	 */
	Sector GetSelection() const {return m_Selection;}

	/**
	 * Setzt die fr nachfolgend hinzugefgte Auenposten zu verwendende RangeMap.
	 */
	void SetRangeMap(unsigned char rangeMap[], char w, char h, char x0, char y0);

	/**
	 * Fgt im angegebenen Sektor einen Auenposten hinzu.
	 */
	void AddBase(const Sector& sector, BYTE propTech);

	/**
	 * @return <code>TRUE</code>, wenn im angegebenen Sektor ein Auenposten hinzugefgt wurde, sonst <code>FALSE</code>.
	 */
	BOOL IsBase(const Sector &sector);

	/**
	 * Gleicht die komplette Rangemap mit der Galaxiemap ab. Gebiete, welche einer Rasse gehren mit der wir einen
	 * Nichtangriffspakt haben, werden von der Rangemap entfernt. bergeben werden dafr ein Zeiger auf alle
	 * Sektoren <code>sectors</code> und ein Wahrheitswert <code>races</code> fr alle Rassen, ob wir einen
	 * Nichtangriffspakt mit dieser Rasse haben.
	 */
	void SynchronizeWithMap(CSector sectors[][STARMAP_SECTORS_VCOUNT], std::set<CString>* races);

	/**
	 * Fhrt fr gefhrliche Anomalien mathematische Gewichte hinzu, so dass dieser Sektor bei der automatischen
	 * Wegsuche nicht berflogen wird.
	 */
	static void SynchronizeWithAnomalies(CSector sectors[][STARMAP_SECTORS_VCOUNT]);

	/**
	 * Lscht alle Basen und setzt die Eintrge in der Rangemap wieder auf Ausgangswert
	 */
	void ClearAll();

	/**
	 * Berechnet den krzesten Weg von position nach target innerhalb der durch range angegebenen Reichweite. Liefert den
	 * kompletten Weg in path zurck. Rckgabewert ist der entsprechend speed als nchstes zu erreichende Punkt auf
	 * dem berechneten Weg. Tritt ein Fehler auf (z.B. wenn das Ziel nicht erreichbar ist), liefert die Funktion ein
	 * Sector-Objekt, dessen Koordinaten auf -1 gesetzt sind. Die in path bereits vorhandenen Eintrge werden gelscht,
	 * die Gre des Arrays wird auf die Lnge des gelieferten Weges gesetzt.
	 */
	Sector CalcPath(const Sector &position, const Sector &target, unsigned char range,
		unsigned char speed, CArray<Sector> &path);

	/**
	 * Fgt der Liste der bevorzugten Ausbreitungsrichtungen ein weiteres Ziel hinzu. Dient der Ermittlung eines Sektors,
	 * in dem ein Auenposten gebaut werden sollte, um sich dem Ziel zu nhern; auch bei weiter entfernten Zielen.
	 */
	void AddTarget(const Sector &target);

	/**
	 * @return <code>TRUE</code>, wenn der angegebene Sektor als Ziel hinzugefgt wurde, sonst <code>FALSE</code>.
	 */
	BOOL IsTarget(const Sector &sector);

	/**
	 * Fgt der Liste der der KI bekannten Systeme ein weiteres System hinzu. Es wird bevorzugt versucht, diese Systeme
	 * in die gegebene Reichweite aufzunehmen, wenn sie dem bereits erschlossenen Gebiet nahe sind (etwas weniger aggresiv
	 * als die Ausbreitung zu einem Ziel).
	 */
	void AddKnownSystem(const Sector &sector);

	/// @return <code>TRUE</code>, wenn sich im angegebenen Sektor ein bekanntes System befindet, sonst <code>FALSE</code>.
	BOOL IsKnownSystem(const Sector &sector);

	/**
	 * Funktion erstellt anhand von Sektoren eine Matrix mit Werten, welche zur Berechnung der besten Position fr
	 * einen Auenposten genutzt werden soll. bergeben werden dafr ein Zeiger auf alle
	 * Sektoren <code>sectors</code> und die Rasse, zu welcher das Starmap Objekt gehrt <code>race</code>.
	 */
	void SetBadAIBaseSectors(CSector sectors[][STARMAP_SECTORS_VCOUNT], const CString& race);

	/**
	 * Ermittelt einen Sektor, in dem gnstig ein Auenposten gebaut werden knnte. Liefert <code>Sector(-1, -1)</code> im
	 * Feld <code>position</code> und <code>-1</code> fr <code>points</code>, wenn nirgendwo ein Auenposten gebaut werden
	 * kann oder dieses Objekt mit <code>bAICalculation == FALSE</code> erzeugt wurde.
	 *
	 * @param variance maximal mgliche Abweichung gegenber der besten Bewertung; aus [0, 1], 0: keine Abweichung,
	 * 1: 100% Abweichung mglich
	 */
	BaseSector CalcAIBaseSector(double variance = 0.1);

	/**
	 * @return Gesamtbewertung des angegebenen Sektors. Zuvor mssen die Funktionen zur Neuberechnung der Bewertung aufgerufen worden sein.
	 */
	short GetPoints(const Sector &sector);

private:
	/// Diese Funktion berechnet die Reichweitenkarte anhand der aktuellen Techstufe <code>propTech</code> und schreibt
	/// das Ergebnis in den Parameter <code>rangeMap</code>. Zustzlich werden Referenzen auf paar Hilfsvariablen
	/// bergeben.
	void CalcRangeMap(BYTE propTech);
	
	/// @return Eintrag der gesetzten RangeMap an der Stelle (x, y), wobei (0, 0) die Auenpostenposition ist
	unsigned char GetRangeMapValue(char x, char y);

	/// Berechnet die Gebietszuwchse neu.
	void RecalcRangePoints();

	/// Berechnet die Bewertungen zum Verbinden benachbarter Gebiete und Verhindern von Lcken.
	void RecalcConnectionPoints();

	/// Berechnet Bewertungen fr bevorzugte Ausbreitungsrichtungen.
	void RecalcTargetPoints();
};

#endif // !defined(AFX_STARMAP_H__28F3FFEA_139A_45A5_AFF3_76F41E48496B__INCLUDED_)
