/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "afx.h"
#include "Options.h"
#include "BuildingInfo.h"
#include "Ships\ShipInfo.h"
#include "Troops\TroopInfo.h"
#include "Galaxy\ResourceRoute.h"

// forward declaration
class CSystem;
class CResearchInfo;

class CAssemblyList : public CObject
{
public:
	DECLARE_SERIAL (CAssemblyList)
	
	// Standardkonstruktor
	CAssemblyList(void);

	// Destruktor
	virtual ~CAssemblyList(void);

	// Kopierkonstruktor
	CAssemblyList(const CAssemblyList & rhs);
	
	// Zuweisungsoperatur
	CAssemblyList & operator=(const CAssemblyList &);

	// Die Serialisierungsfunktion
	virtual void Serialize(CArchive &ar);

// Zugriffsfunktionen
	// zum Lesen der Membervariablen
	// Funktion gibt den Bauauftrag an der Stelle "pos" in der Bauliste zurck
	short GetAssemblyListEntry(USHORT pos) const {return m_iEntry[pos];}
	
	// Nachfolgende Funktionen geben die noch verbleibenden Kosten des Auftrages in der Bauliste zurck
	int GetNeededIndustryInAssemblyList(USHORT entry) const {return m_iNeededIndustryInAssemblyList[entry];}
	UINT GetNeededTitanInAssemblyList(USHORT entry) const {return m_iNeededTitanInAssemblyList[entry];}
	UINT GetNeededDeuteriumInAssemblyList(USHORT entry) const {return m_iNeededDeuteriumInAssemblyList[entry];}
	UINT GetNeededDuraniumInAssemblyList(USHORT entry) const {return m_iNeededDuraniumInAssemblyList[entry];}
	UINT GetNeededCrystalInAssemblyList(USHORT entry) const {return m_iNeededCrystalInAssemblyList[entry];}
	UINT GetNeededIridiumInAssemblyList(USHORT entry) const {return m_iNeededIridiumInAssemblyList[entry];}
	UINT GetNeededDeritiumInAssemblyList(USHORT entry) const {return m_iNeededDeritiumInAssemblyList[entry];}

	/// Funktion gibt die jeweiligen verbleibenden Kosten fr einen Bauauftrag zurck.
	UINT GetNeededResourceInAssemblyList(USHORT entry, BYTE res) const;

	// Nachfolgende Funktionen geben die Kosten fr einen Bauauftrag zurck
	UINT GetNeededIndustryForBuild() const {return m_iNeededIndustryForBuild;}
	UINT GetNeededTitanForBuild() const {return m_iNeededTitanForBuild;}
	UINT GetNeededDeuteriumForBuild() const {return m_iNeededDeuteriumForBuild;}
	UINT GetNeededDuraniumForBuild() const {return m_iNeededDuraniumForBuild;}
	UINT GetNeededCrystalForBuild() const {return m_iNeededCrystalForBuild;}
	UINT GetNeededIridiumForBuild() const {return m_iNeededIridiumForBuild;}
	UINT GetNeededDeritiumForBuild() const {return m_iNeededDeritiumForBuild;}

	/// Funktion gibt die jeweiligen Kosten fr einen Bauauftrag zurck.
	UINT GetNeededResourceForBuild(BYTE res) const;

	// Funktion gibt zurck, ob das Gebude in der Bauliste gekauft wurde
	BOOLEAN GetWasBuildingBought() const {return m_bWasBuildingBought;}
	
	// Funktion gibt die kompletten Kaufkosten des Bauauftrages zurck
	int GetBuildCosts() const {return m_iBuildCosts;}
	
	// zum Schreiben der Membervariablen
	// Setzt ob ein Bauauftrag gekauft wurde auf wahr oder falsch
	void SetWasBuildingBought(BOOLEAN TrueOrFalse) {m_bWasBuildingBought = TrueOrFalse;}

// sonstige Funktionen
	// Funktion berechnet die bentigten Rohstoffe. bergeben wird die Information des gewnschten Gebudes bzw. des
	// gewnschten Schiffes oder der gewnschten Truppe, der RunningNumber des Bauauftrages, einen Zeiger auf alle
	// vorhandenen Gebude in dem System und ein Zeiger auf die Forschung
	void CalculateNeededRessources(CBuildingInfo* buildingInfo, CShipInfo* shipInfo, CTroopInfo* troopInfo,
		BuildingArray* m_Buildings, int RunningNumber, CResearchInfo* ResearchInfo, float modifier = 1.0f);

	// Funktion berechnet die Kosten fr jedes Update in der Bauliste. Wird in der NextRound() Funktion aufgerufen.
	// Weil in der Zwischenzeit htte der Spieler ein Gebude bauen knnen, das die Kosten mancher Updates verndern
	// knnte. bergeben wird ein Zeiger auf das Feld aller Gebudeinformationen, ein Zeiger auf alle stehenden
	// Gebude im System und ein Zeiger auf die Forschungsinformation.
	void CalculateNeededRessourcesForUpdate(BuildingInfoArray* follower, BuildingArray* m_Buildings, CResearchInfo* ResearchInfo);

	// Funktion	macht einen Eintrag in der Bauliste, dafr wird die RunningNumber des Auftrages bergeben.
	// z.B. 1 = primitive Farm, -2 = Upgrade primitive Farm zu Typ 1 Automatikfarm.
	// Nachdem wir diese Funktion aufgerufen haben und ein TRUE zurckbekommen haben mssen wir die Funktion 
	// CalculateVariables() aufgerufen! (Weil wir wenn die Handelsgter aus der Bauliste verschwinden, sofort das
	// neue Credits angezeigt werden soll.)
	BOOLEAN MakeEntry(int runningNumber, const CPoint &ko, CSystem systems[][STARMAP_SECTORS_VCOUNT], bool bOnlyTest = false);

	// Funktion berechnet die Kosten des Bauauftrags, wenn man dieses sofort kaufen will. Die Kosten des Sofortkaufes
	// sind von den Marktpreisen abhngig, daher mssen diese bergeben werden.
	void CalculateBuildCosts(USHORT resPrices[5]);

	// Funktion setzt die noch restlichen Baukosten auf 1 und sagt, dass wir jetzt was gekauft haben. Wenn wir kaufen
	// knnen bestimmt die Fkt "CalculateBuildCosts()". Diese Fkt. immer vorher aufrufen. Die Creditskosten werden
	// zurckgegeben. bergeben wird das aktuelle Credits des Imperiums.
	int BuyBuilding(int EmpiresCredits);

	// Die Funktion berechnet was nach dem Tick noch an verbleibender Industrieleistung aufzubringen ist.
	// Ist der Bauauftrag fertig gibt die Funktion ein TRUE zurck. Wenn wir ein TRUE zurckbekommen, mssen
	// wir direkt danach die Funktion ClearAssemblyList() aufrufen!
	BOOLEAN CalculateBuildInAssemblyList(USHORT m_iIndustryProd);

	// Funktion lscht einen Eintrag aus der Bauliste, wenn das Gebude fertig wurde oder wir den ersten
	// Eintrag manuell lschen mchten. Nach Aufruf dieser Funktion mu unbedingt die Funktion
	// CalculateVariables() aufgerufen werden.
	void ClearAssemblyList(const CPoint &ko, CSystem systems[][STARMAP_SECTORS_VCOUNT]);

	// Ordnet die Bauliste so, dass keine leeren Eintrge in der Mitte vorkommen knnen. Wird z.B. aufgerufen,
	// nachdem wir einen Auftrag aus der Bauliste entfernt haben. Darf aber nicht aufgerufen werden, wenn wir
	// den 0. Eintrag entfernen, dann mssen wir ClearAssemblyList() aufrufen, weil diese Funktion die ntigen
	// Rohstoffe gleich mit abzieht.
	void AdjustAssemblyList(short entry);

	// Resetfunktion fr die Klasse CAssemblyList
	void Reset();

private:
// private Funktionen
	/// Diese Funktion entfernt die bentigten Ressourcen aus dem lokalen Lager des Systems und falls Ressourcenrouten
	/// bestehen auch die Ressourcen in den Startsystemen der Route. Aber nur falls dies auch notwendig sein sollte.
	void RemoveResourceFromStorage(BYTE res, const CPoint &ko, CSystem systems[][STARMAP_SECTORS_VCOUNT], CArray<CPoint>* routesFrom);

// Attribute
	// Der Auftrag in der Bauliste.
	short m_iEntry[ALE];
	
	// Variablen geben die noch verbleibenden Kosten der Elemente in der Bauliste an
	int	  m_iNeededIndustryInAssemblyList[ALE];	// bentigte IP die wir noch bentigen um das Projekt fertig zu stellen
	UINT m_iNeededTitanInAssemblyList[ALE];	// bentigtes Titan, das wir bentigen um das Projekt fertig zu stellen
	UINT m_iNeededDeuteriumInAssemblyList[ALE];// bentigtes Deuterium, das wir bentigen um das Projekt fertig zu stellen
	UINT m_iNeededDuraniumInAssemblyList[ALE];	// bentigtes Duranium, das wir bentigen um das Projekt fertig zu stellen
	UINT m_iNeededCrystalInAssemblyList[ALE];	// bentigtes Crystal, das wir bentigen um das Projekt fertig zu stellen
	UINT m_iNeededIridiumInAssemblyList[ALE];	// bentigtes Iridium, das wir bentigen um das Projekt fertig zu stellen
	UINT m_iNeededDeritiumInAssemblyList[ALE];// bentigtes Deritium, das wir bentigen um das Projekt fertig zu stellen

	// Variablen, die Angeben, wieviel Industrie und Rohstoffe zum Bau bentigt werden
	UINT m_iNeededIndustryForBuild;			// bentigte IP zum Bauen des Gebudes/Updates
	UINT m_iNeededTitanForBuild;				// bentigtes Titan zum Bauen des Gebudes/Updates
	UINT m_iNeededDeuteriumForBuild;			// bentigtes Deuterium zum Bauen des Gebudes/Updates
	UINT m_iNeededDuraniumForBuild;			// bentigtes Duranium zum Bauen des Gebudes/Updates
	UINT m_iNeededCrystalForBuild;				// bentigtes Crystal zum Bauen des Gebudes/Updates
	UINT m_iNeededIridiumForBuild;				// bentigtes Iridium zum Bauen des Gebudes/Updates
	UINT m_iNeededDeritiumForBuild;			// bentigtes Deritium zum Bauen des Gebudes/Updates

	// Wurde das Gebude gekauft in dieser Runde gekauft
	BOOLEAN   m_bWasBuildingBought;			
	
	// Die Baukosten eines Auftrages
	int    m_iBuildCosts;	
};
