/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include <map>
#include <list>

class CGlobalBuildings : public CObject
{
public:
	DECLARE_SERIAL (CGlobalBuildings)
	// Standardkonstruktor
	CGlobalBuildings(void);

	// Destruktor
	~CGlobalBuildings(void);

	// Die Serialisierungsfunktion
	virtual void Serialize(CArchive &ar);

	// Zugriffsfunktionen
	// zum Lesen der Membervariablen
	/// Funktion gibt die Anzahl eines bestimmtes Gebudes fr eine bestimmte Major-Rasse zurck.
	/// @param sRaceID Rassen-ID
	/// @param nID Gebude-ID
	/// @return Anzahl der Gebude
	int GetCountGlobalBuilding(const CString& sRaceID, USHORT nID) const;
	
	// zum Schreiben der Membervariablen
	/// Funktion fgt ein Gebude fr eine bestimmte Rasse den globalen Gebuden hinzu.
	/// @param sRaceID Rassen-ID
	/// @param nID Gebude-ID
	void AddGlobalBuilding(const CString& sRaceID, USHORT nID) {m_mGlobalBuildings[sRaceID].push_back(nID);}

	// sonstige Funktionen
	/// Funktion lscht ein globales Gebude aus dem Feld der globalen Gebude. Diese Funktion sollte aufgerufen
	/// werden, wenn ein solches Gebude abgerissen oder verloren wird.
	/// @param sRaceID Rassen-ID
	/// @param nID Gebude-ID
	void DeleteGlobalBuilding(const CString& sRaceID, USHORT nID);

	// Resetfunktion fr die Klasse CGlobalBuildings
	void Reset();

private:
	// Dieses Feld beinhaltet die Gebude, welche ein imperienweites Attribut haben und in einer Bauliste stehen, sowei
	// alle Gebude die in allen Systemen stehen. Kommen mehrere gleiche Gebude vor, so ist deren ID auch mehrmals
	// hier in dem Feld vorhanden.
	std::map<CString, std::list<USHORT> > m_mGlobalBuildings;	
};
