/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
// System.h: Schnittstelle fr die Klasse CSystem.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SYSTEM_H__52476A8D_3AFC_4EFC_B456_155002572D31__INCLUDED_)
#define AFX_SYSTEM_H__52476A8D_3AFC_4EFC_B456_155002572D31__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "..\Galaxy\Sector.h"
#include "Worker.h"
#include "AssemblyList.h"
#include "SystemProd.h"
#include "GlobalBuildings.h"
#include "Trade\TradeRoute.h"
#include "array_sort.h"

// forward declaration
class CMajor;
class CMinor;
class CBotf2Doc;

class CSystem : public CObject  
{
public:
	DECLARE_SERIAL (CSystem)
	// Standardkonstruktor
	CSystem(void);

	// Destruktor
	virtual ~CSystem(void);
	
	// Kopierkonstruktor
	CSystem(const CSystem & rhs);
	
	// Zuweisungsoperatur
	CSystem & operator=(const CSystem &);
	
	// Serialisierungsfunktion
	virtual void Serialize(CArchive &ar);

// Zugriffsfunktionen
	// zum Lesen der Membervariablen
	// Funktion gibt den Besitzer des Systems zurck, sollte eigentlich immer auch der Besitzer des Sektors sein.	
	const CString& GetOwnerOfSystem() const {return m_sOwnerOfSystem;}
		
	// Funktion gibt die aktuelle Bevlkerung des Systems zurck.
	double GetHabitants() const {return m_dHabitants;}
	
	// Funktion gibt einen Zeiger auf die Bauliste des Systems zurck.
	CAssemblyList* GetAssemblyList() {return &m_AssemblyList;}

	/// Funktion berechnet die theoretisch bentigte Anzahl an Runden, bis ein beliebiges Projekt in
	/// diesem System fertig sein wird.
	/// @param nID ID des Projektes
	/// @return Anzahl der bentigten Runden
	int GetNeededRoundsToCompleteProject(int nID);

	// Funktion gibt einen Zeiger auf alle Produktionswerte und manche Boni des Systems zurck
	CSystemProd* GetProduction() {return &m_Production;}
	
	// Funktion gibt einen Zeiger auf das Feld aller Gebude im System zurck.
	BuildingArray* GetAllBuildings() {return &m_Buildings;}
	
	// Funktionen geben jeweils einen Zeiger auf das Feld mit den Informationen ber baubare Gebude,
	// baubare Updates oder baubaren Schiffen in dem System zurck.
	CArray<short,short>* GetBuildableBuildings() {return &m_BuildableBuildings;}
	CArray<short,short>* GetBuildableUpdates() {return &m_BuildableUpdates;}
	CArray<short,short>* GetBuildableShips() {return &m_BuildableShips;}
	CArray<BYTE,BYTE>*   GetBuildableTroops() {return &m_BuildableTroops;}

	// Funktion gibt die Anzahl oder die RunningNumber (ID) der Gebude zurck, welche Arbeiter bentigen.
	// Wir bergeben dafr als Parameter den Typ des Gebudes (FARM, BAUHOF usw.) und einen Modus.
	// Ist der Modus NULL, dann bekommen wir die Anzahl zurck, ist der Modus EINS, dann die RunningNumber.
	USHORT GetNumberOfWorkbuildings(int WhatWorkbuilding, int Modus, BuildingInfoArray* buildingInfos) const;

	// Funktion gibt die Anzahl des Gebudes mit der bergebenen RunningNumber zurck.
	USHORT GetNumberOfBuilding(USHORT runningNumber) const;

	// Funktion gibt die Anzahl der aktiven Arbeiter zurck, deren Typ als Parameter an die Funktion bergeben wurde.
	USHORT GetWorker(int WhatWorker) const {return m_Workers.GetWorker(WhatWorker);}

	// Funktion gibt einen Zeiger auf das Arbeiterobjekt der Systemklasse zurck.
	CWorker* GetWorker() {return &m_Workers;}

	// Funktion gibt die aktuelle Moral im System zurck
	short GetMoral() const {return m_iMoral;}

	/// Funktion gibt den Prozentwert der Blockade des Systems zurck. Ist dieser Rckgabewert gleich
	/// <code>NULL</code>, so existiert keine Blockade im System.
	BYTE GetBlockade() const {return m_byBlockade;}

	// Funktionen geben die einzelnen Lagerinhalte des Systems zurck.
	long GetFoodStore() const {return m_iFoodStore;}
	UINT GetTitanStore() const {return m_iTitanStore;}
	UINT GetDeuteriumStore() const {return m_iDeuteriumStore;}
	UINT GetDuraniumStore() const {return m_iDuraniumStore;}
	UINT GetCrystalStore() const {return m_iCrystalStore;}
	UINT GetIridiumStore() const {return m_iIridiumStore;}
	UINT GetDeritiumStore() const {return m_iDeritiumStore;}
	
	// Funktion gibt den Lagerinhalt der Ressource zurck, die an die Funktion bergeben wurde.
	UINT GetResourceStore(USHORT res) const;
	
	// Funktion gibt einen Zeiger auf den Lagerinhalt der Ressource zurck, die an die Funktion bergeben wurde.
	UINT* GetResourceStorages(USHORT res);

	// Funktion gibt die Anzahl der Gebude mit der bergebenen RunningNumber wieder, welche nchste Runde
	// abgerissen werden sollen
	USHORT GetBuildingDestroy(int RunningNumber);

	/// Funktion gibt einen Zeiger auf die Handelsrouten von diesem System aus zurck
	CArray<CTradeRoute>* GetTradeRoutes() {return &m_TradeRoutes;}

	/// Funktion gibt einen Zeiger auf die Ressourcenrouten, welche in dieses System fhren zurck.
	CArray<CResourceRoute>* GetResourceRoutes() {return &m_ResourceRoutes;}

	/// Funktion gibt einen Zeiger auf die stationierten Truppen in diesem System zurck.
	CArray<CTroop>* GetTroops() {return &m_Troops;}

	/// Funktion gibt zurck, ob die Autobaufunktion in dem System aktiviert ist.
	BOOLEAN GetAutoBuild() const {return m_bAutoBuild;}
		
	// zum Schreiben der Membervariabblen
	// Funktion setzt den neuen Besitzer des Systems. bergeben wird der Besitzer.
	void SetOwnerOfSystem(const CString& sOwnerOfSystem);
	
	// Funktion setzt die Bevlkerungsanzahl des Systems. bergeben wird die Bevlkerung aller Planeten des Sektors.
	// Gleichzeitig berprft die Funktion auch, ob man eine weitere Handelsroute aufgrund der Bevlkerung bekommt, dann
	// gibt die Funktion ein <code>TRUE</code> zurck, ansonsten <code>FALSE</code>.
	BOOLEAN SetHabitants(double habitants);
	
	// Funktion sagt ob ein bestimmtes Gebude in dem System baubar ist oder nicht. Als Parameter werden dafr
	// die RunningNumber des gewnschten Gebudes und der Wert bergeben.
	void SetBuildableBuildings(int RunningNumber, BOOLEAN TrueOrFalse);
	
	// Funktion sagt ob ein bestimmtes Gebudeupdate in dem System baubar ist oder nicht. Als Parameter werden dafr
	// die RunningNumber des gewnschten Gebudes und der Wert bergeben.
	void SetBuildableUpdates(int RunningNumber, BOOLEAN TrueOrFalse);

	/// Funktion setzt das Gebude aus dem Feld aller Gebude im System <code>m_Buildings</code> mit dem Index
	/// <code>index</code> online oder offline.
	/// @param index Feldeintrag in <code>m_Buildings</code>
	/// @param newStatus neuer Status des Gebudes - <code>1</code> fr online, ansonsten <code>NULL</code>
	void SetIsBuildingOnline(int index, BOOLEAN newStatus);

	// Komplette Zugriffsfunktion fr das Arbeiterobjekt. Bei Modus 0 wird der "WhatWorker" inkrementiert, bei Modus 2 wird
	// er dekrementiert und bei Modus 2 wird der "WhatWorker" auf den Wert von Value gesetzt.
	void SetWorker(int WhatWorker, int Value, int Modus);
	
	// Funktion setzt alle vorhandenen Arbeiter soweit wie mglich in Gebude, die Arbeiter bentigen.
	void SetWorkersIntoBuildings();
	
	// Funktion addiert moralAdd zu m_iMoral dazu und mach gleichzeitig noch die berprfen auf den richtigen Bereich.
	void SetMoral(short moralAdd) {if ((m_iMoral+moralAdd) >= 0) m_iMoral += moralAdd; if (m_iMoral > 200) m_iMoral = 200;}

	/// Funktion setzt den Prozentwert der Blockade fest. Ist dieser Wert grer als <code>NULL</code>, so besteht
	/// eine Blockade.
	void SetBlockade(BYTE blockadeValue) {m_byBlockade = min(blockadeValue, 100);}

	// Funktion setzt das Nahrungslager des Systems auf den Parameterwert "food".
	void SetFoodStore(ULONG food) {m_iFoodStore = food;}

	// Funktionen addieren den bergebenen Paramterwert auf das jeweilige Ressourcenlager.
	void SetTitanStore(int titanAdd) {m_iTitanStore += titanAdd;}
	void SetDeuteriumStore(int deuteriumAdd) {m_iDeuteriumStore += deuteriumAdd;}
	void SetDuraniumStore(int duraniumAdd) {m_iDuraniumStore += duraniumAdd;}
	void SetCrystalStore(int crystalAdd) {m_iCrystalStore += crystalAdd;}
	void SetIridiumStore(int iridiumAdd) {m_iIridiumStore += iridiumAdd;}
	void SetDeritiumStore(int deritiumAdd) {m_iDeritiumStore += deritiumAdd;}
	
	// Funktion addiert resAdd zur Ressource res.
	void SetResourceStore(USHORT res, int resAdd);
	
	// Diese Funktion setzt die abzureienden Gebude fest. Die zu bergebenden Parameter sind die RunningNumber
	// des Gebudes und einen Wert 1 fr hinzufgen und 0 fr wieder aus der Liste nehmen
	void SetBuildingDestroy(int RunningNumber, BOOLEAN add);

	/// Funktion setzt fest, ob die Autobaufunktion in dem System aktiviert ist.
	/// param is <code>TRUE</code> wenn eingeschalten, ansonsten <code>FALSE</code>
	void SetAutoBuild(BOOLEAN is) {m_bAutoBuild = is;}
	
// sonstige Funktionen	
	// Funktion berechnet aus den Eigenschaften der stehenden Gebude alle Attribute der Systemklasse.
	void CalculateVariables(BuildingInfoArray*, CResearchInfo*, CArray<CPlanet>*, CMajor* pOwner, const CString*);

	// Funktion berechnet die Lagerinhalte des Systems. Aufrufen bei Ende bzw. Beginn einer neuen Runde.
	// Gibt die Funktion TRUE zurck hat sich das System Aufgrund zu schlechter Moral vom Besitzer losgesagt.
	BOOLEAN CalculateStorages(CResearchInfo* researchInfo, int diliAdd);
	
	// Funktion fgt ein neues Gebude zum System hinzu.
	void AddNewBuilding(CBuilding &building) { m_Buildings.Add(building); }

	// Funktion lscht alle Gebude, die die bergebene RunningNumber haben und gibt deren Anzahl zurck.
	// -> Danach mu AddBuilding() mit dem Nachfolger gleich der Anzahl aufgerufen werden.
	int UpdateBuildings(int RunningNumber);
	
	// Funktion reit alle Gebude ab, die in der Variable m_BuildingDestroy stehen. Funktion wird in der Doc 
	// bei NextRound() aufgerufen.
// --- wird noch berarbeitet
	bool DestroyBuildings(void);
	
	// Funktion berechnet die baubaren Gebude und Gebudeupdates in dem System.
	void CalculateBuildableBuildings(CSector* sector, BuildingInfoArray* buildingInfo, CMajor* pMajor, CGlobalBuildings* globals);
	
	// Funktion berechnet die baubaren Schiffe in dem System.
	void CalculateBuildableShips(CBotf2Doc* pDoc, const CPoint& p);

	// Diese Funktion berechnet die baubaren Truppen in diesem System
	void CalculateBuildableTroops(const CArray<CTroopInfo>* troopInfos, const CResearch *research);
	
	// Funktion berechnet die Anzahl aller Farmen, Bauhfe usw., also alle Gebude die Arbeiter bentigen.
	// Sie mu am Rundenanfang vor CalculateVariables() aufgerufen werden und sortiert gleichzeitig das
	// CArray m_Buildings nach der RunningNumber. // In der Doc-Klasse nach der Funktion DestroyBuildings()
	// und zu Beginn aufrufen!
	void CalculateNumberOfWorkbuildings(BuildingInfoArray *buildingInfos);

	// Funktion berechnet die imperiumweite Moralproduktion, welche aus diesem System generiert wird.
	void CalculateEmpireWideMoralProd(BuildingInfoArray *buildingInfos);
	
	// Funktion setzt das letzte Gebude, welches gebaut wurde online, sofern dies mglich ist.
	int SetNewBuildingOnline(BuildingInfoArray *buildingInfos);
	
	// Funktion berprft Gebude die Energie bentigen und schaltet diese gegebenfalls ab,
	// falls zuwenig Energie im System vorhanden ist. Diese Funktion aufrufen, bevor wir CalculateVariables() usw.
	// aufrufen, weil wir ja die bsen Onlinegebude vorher ausschalten wollen.
	bool CheckEnergyBuildings(const BuildingInfoArray* pvBuildingInfos);

	// Funktion fgt einen Baulistencheck durch
	BOOLEAN AssemblyListCheck(BuildingInfoArray* buildingInfo, CGlobalBuildings* globals, int ID = 0);

	// Funktion baut die Gebude der Minorrace, wenn wir eine Mitgliedschaft mit dieser erreicht haben.
	void BuildBuildingsForMinorRace(CSector* sector, BuildingInfoArray* buildingInfo, USHORT averageTechlevel, const CMinor* pMinor);
	
	// Funktion berechnet und baut die Startgebude in einem System, nachdem wir einen Planeten
	// in diesem kolonisiert haben.
	void BuildBuildingsAfterColonization(CSector* sector, BuildingInfoArray* buildingInfo, USHORT ColonizationPoints);

	/// Diese Funktion entfernt alle speziellen Gebude aus der Gebudeliste. Diese Funktion sollte nach Eroberung des Systems
	/// aufgerufen werden. Danach sind keine Gebude mehr vorhanden, die nur x mal pro Imperium baubar gewesen oder die nur die Rasse
	/// selbst bauen darf.
	/// @param pvBuildingInfos Zeiger auf den Vektor mit allen Gebudeinformationen
	void RemoveSpecialRaceBuildings(const BuildingInfoArray* pvBuildingInfos);

	// Funktion berprft, ob wie aufgrund der Bevlkerung hier im System berhaupt (noch) eine Handelsroute
	// anlegen knnen
	BOOLEAN CanAddTradeRoute(CResearchInfo* researchInfo);
	
	/// Funktion generiert eine neue Handelsroute. Wenn die Funktion <code>TRUE</code> zurckgibt, dann konnte die
	/// Handelsroute erfolgreich angelegt werden. Als Parameter wird dabei die Koordinate <code>dest</code>des
	/// Zielsektors bergeben sowie ein Zeiger auf alle Systeme <code>systems</code> auf der Map.
	BOOLEAN AddTradeRoute(CPoint dest, CSystem systems[][STARMAP_SECTORS_VCOUNT], CResearchInfo* researchInfo);

	// Funktion gibt das gesamte Credits zurck, was alle Handelsrouten aus diesem System generiert haben.
	USHORT CreditsFromTradeRoutes();

	// Funktion berprft alle Handelsrouten in dem System, ob sie Aufgrund der Bevlkerung oder der Gebude noch
	// da sein drfen. Wurden Handelsrouten gelscht, so gibt die Funktion die Anzahl der Lschungen zurck
	BYTE CheckTradeRoutes(CResearchInfo* researchInfo);

	/// Funktion generiert eine neue Ressourcenroute. Wenn die Funktion <code>TRUE</code> zurckgibt, dann konnte die
	/// Ressourcenroute erfolgreich angelegt werden. Als Parameter wird dabei die Koordinate <code>dest</code> des
	/// Zielsektors bergeben sowie die Art der betroffenen Ressource <code>res</code> und einen Zeiger auf alle
	/// Systeme <code>systems</code> auf der Map.
	BOOLEAN AddResourceRoute(CPoint dest, BYTE res, CSystem systems[][STARMAP_SECTORS_VCOUNT], CResearchInfo* researchInfo);

	/// Funktion berprft alle Ressourcenrouten in dem System, ob sie Aufgrund der Bevlkerung oder der Gebude noch
	/// da sein drfen. Wurden Ressourcenrouten gelscht, so gibt die Funktion die Anzahl der Lschungen zurck.
	BYTE CheckResourceRoutes(CResearchInfo* researchInfo);

	/// Diese Funktion fgt eine neue Truppe <code>troop</code> dem System hinzu.
	void AddTroop(const CTroop* troop) {m_Troops.Add(*troop);}

	/// Wenn in diesem System Truppen stationiert sind, dann wird deren Moralwert mit einbezogen.
	/// Ist die Moral im System unter 100, so wird der Moralwert der Einheit dazuaddiert, wenn er ber 100 ist, dann wird
	/// der Moralwert abgezogen.
	void IncludeTroopMoralValue(CArray<CTroopInfo>* troopInfo);
	
	// Resetfunktion fr die Klasse CSystem.
	void ResetSystem();

private:
	// Der Besitzer des Systems
	CString m_sOwnerOfSystem;
	
	// Einwohner in dem System
	double m_dHabitants;
	
	// Die Bauliste des Systems
	CAssemblyList m_AssemblyList;

	// Die Produktion des Systems
	CSystemProd m_Production;

	// Alle Gebude in dem System
	BuildingArray m_Buildings;

	// Variable beinhaltet welche Gebude in dem System baubar sind
	CArray<short,short> m_BuildableBuildings;
	
	// Variable beinhaltet welche Gebudeupdates in dem System baubar sind
	CArray<short,short> m_BuildableUpdates;

	// Variable beinhaltet welche Schiffe in dem System baubar sind
	CArray<short,short> m_BuildableShips;

	// Variable beinhaltet welche Truppen in dem Sytem baubar sind
	CArray<BYTE,BYTE> m_BuildableTroops;

	// Variable beinhaltet welche Gebude(-stufe) in dem System immer baubar ist
	CArray<short,short> m_AllwaysBuildableBuildings;

	// Variable beinhaltet welche Gebude und Updates wir in dem System bauen knnen, bevor der Baulistencheck
	// gemacht wurde. Diese Variable mu von auerhalb nie betrachtet werden
	CArray<short,short> m_BuildableWithoutAssemblylistCheck;

	// Die Arbeiter in dem System
	CWorker m_Workers;
	
	// Die Moral der Bevlkerung auf in dem System
	short m_iMoral;

	/// Besteht in dem System eine feindliche Blockade, so gibt diese Variable den Prozentwert der Blockade an.
	/// Ist dieser grer <code>NULL</code>, so besteht eine Blockade.
	BYTE m_byBlockade;

	// Die Lagerinhalt der einzelnen Rohstoffe und der Nahrung
	int  m_iFoodStore;					// Lager fr Nahrung
	UINT m_iTitanStore;					// Lager fr Titan
	UINT m_iDeuteriumStore;				// Lager fr Deuterium
	UINT m_iDuraniumStore;				// Lager fr Duranium
	UINT m_iCrystalStore;				// Lager fr Kristalle
	UINT m_iIridiumStore;				// Lager fr Iridium
	UINT m_iDeritiumStore;				// Lager fr Deritium

	// Variable zum Abreien von Gebuden
	CArray<USHORT,USHORT> m_BuildingDestroy;
	
	// Anzahl der Gebude des jeweiligen Types
	BYTE m_iFoodBuildings;				// Anzahl der Nahrungsgebude in dem System
	BYTE m_iIndustryBuildings;			// Anzahl der Industriegebude in dem System
	BYTE m_iEnergyBuildings;			// Anzahl der Energiegebude in dem System
	BYTE m_iSecurityBuildings;			// Anzahl der Geheimdienstgebude in dem System
	BYTE m_iResearchBuildings;			// Anzahl der Forschungsgebude in dem System
	BYTE m_iTitanMines;					// Anzahl der Titanminen in dem System
	BYTE m_iDeuteriumMines;				// Anzahl der Deuteriumminen in dem System
	BYTE m_iDuraniumMines;				// Anzahl der Duraniumminen in dem System
	BYTE m_iIridiumMines;				// Anzahl der Iridiumminen in dem System
	BYTE m_iCrystalMines;				// Anzahl der Crystalminen in dem System

	CArray<CTradeRoute> m_TradeRoutes;			///< Die Handelsrouten in dem System

	CArray<CResourceRoute> m_ResourceRoutes;	///< Die Ressourcenrouten in dem System

	// Die maximalen Handelsrouten allein durch die Bevlkerung
	BYTE m_byMaxTradeRoutesFromHab;

	// Die vorhanden Truppen in dem System
	CArray<CTroop> m_Troops;

	/// Autofunktion aktiviert oder nicht
	BOOLEAN m_bAutoBuild;
		
	// private Hilfsfunktionen (mal schauen ob wir die direkt in die cpp-Datei schreiben knnen)
	BOOLEAN CheckTech(CBuildingInfo* building, CResearch* research);
	BOOLEAN CheckPlanet(CBuildingInfo* building, CSector* sector);
	BOOLEAN CheckGeneralConditions(CBuildingInfo* building, CSector* sector, CGlobalBuildings* globals, CMajor* pMajor);
	BOOLEAN CheckFollower(BuildingInfoArray* buildings, USHORT ID, BOOLEAN flag = 0, BOOLEAN equivalence = 0);

};

#endif // !defined(AFX_SYSTEM_H__52476A8D_3AFC_4EFC_B456_155002572D31__INCLUDED_)
