/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "afx.h"
#include "Options.h"
#include "System\System.h"
#include "Galaxy\Sector.h"
#include "Trade\TradeHistory.h"
#include <map>

using namespace std;
// forward declaration
class CMajor;

typedef struct {USHORT res; int number; CPoint system; int price;} m_TradeStruct;

class CTrade : public CObject
{
public:
	DECLARE_SERIAL (CTrade)
	// Standardkonstruktor
	CTrade(void);

	// Destruktor
	virtual ~CTrade(void);

	// Die Serialisierungsfunktion
	virtual void Serialize(CArchive &ar);

// Zugriffsfunktionen
	// zum Lesen der Membervariablen

	/// Funktion gibt die Kursgeschichte zurck.
	/// @return Zeiger auf ein CTradeHistory Objekt
	CTradeHistory* GetTradeHistory(void) {return &m_TradeHistory;}

	// Funktion gibt das Feld der aktuellen Preise der Ressourcen zurck
	USHORT* GetRessourcePrice() {return m_iRessourcePrice;}

	// Funktion gibt das Feld der aktuellen Preise der Ressourcen zurck, so wie sie am
	// Anfang der jeweiligen Runde waren
	USHORT* GetRessourcePriceAtRoundStart() {return m_iRessourcePriceAtRoundStart;}

	// Funktion gibt die aktuelle Steuer auf Handelsaktivitten zurck
	float GetTax() const {return m_fTax;}

	// Funktion gibt das Feld mit den Steuern, die wir nur durch den Sofortkauf bei Bauauftrgen
	// machen, zurck
	USHORT* GetTaxesFromBuying() {return m_iTaxes;}

	// Funktion gibt die Menge, die wir bei einem Klick kaufen/verkaufen zurck
	USHORT GetQuantity() const {return m_iQuantity;}

	// Diese Klassenfunktion gibt den Besitzer eines Monopols fr eine bestimmte Ressource zurck
	static CString GetMonopolOwner(USHORT res) {return m_sMonopolOwner[res];}

	/// Diese Klassenfunktion gibt das statische Feld mit den Monopolbesitzern zurck.
	const static CString* GetMonopolOwner() {return m_sMonopolOwner;}

	// Funktion gibt uns das Feld zurck, in dem steht ob wir ein Monopol gekauft haben
	double* GetMonopolBuying() {return m_dMonopolBuy;}

	// Funktion gibt das Feld der Handelsaktivitten fr das komplette Imperium zurck
	CArray<m_TradeStruct,m_TradeStruct>* GetTradeActions() {return &m_TradeActions;}

	// zum Schreiben der Membervariablen
		
	// Funktion setzt den Preis fr eine Ressource fest
	void SetRessourcePrice(USHORT res, USHORT price) {m_iRessourcePrice[res] = price;}

	// Funktion setzt die Steuer auf Handelsaktivitten fest
	void SetTax(float newTax) {m_fTax = newTax;}

	// Funktion setzt die Menge, die wir pro Handelsaktivitt umsetzen fest
	void SetQuantity(USHORT newQuantity) {m_iQuantity = newQuantity;}

	// Diese Klassenfunktion setzt den Besitzer eines Monopols fr eine bestimmte Ressource
	static void SetMonopolOwner(USHORT res, const CString& sOwnerID) {m_sMonopolOwner[res] = sOwnerID;}

	// Funktion setzt den veranschlagten Kaufpreis einer Ressource in das Feld m_dMonopolBuy
	void SetMonopolBuying(USHORT res, double costs) {m_dMonopolBuy[res] = costs;}

// sonstige Funktionen
	// Funktion kauft die Anzahl der jeweiligen Ressource fr das System und fgt den Auftrag in das Array
	// m_TradeActions ein. Danach berechnet sie den Preis der Ressource nach dem Kauf. Steuern
	// werden hier noch nicht in den Preis mit einbezogen. Der Preis inkl. Steuern wird zurckgegeben.
	// Falls flag == 1, dann wird nix in das Feld m_TradeActions geschrieben. Nur die Preisauswirkungen des Kaufes
	// auf die Brse werden beachtet
	int BuyRessource(USHORT res, ULONG number, CPoint system, long empires_credits, BOOL flag = 0);

	// Funktion verkauft die Anzahl der jeweiligen Ressource aus dem System und fgt den Auftrag in das Array
	// m_TradeActions ein. Danach berechnet sie den Preis der Ressource nach dem Verkauf. Steuern
	// werden hier noch nicht in den Preis mit einbezogen. Falls flag == 1, dann wird nix in das Feld m_TradeActions
	// geschrieben. Nur die Preisauswirkungen des Verkaufes auf die Brse werden beachtet
	void SellRessource(USHORT res, ULONG number, CPoint system, BOOL flag = 0);

	// Funktion berechnet die ganzen Handelsaktionen, lagert also Ressourcen ein oder gibt das Credits, welches
	// wir durch den Verkauf bekommen haben an das jeweilige Imperium
	void CalculateTradeActions(CMajor* pMajor, CSystem systems[][STARMAP_SECTORS_VCOUNT], CSector sectors[][STARMAP_SECTORS_VCOUNT], USHORT* taxes);

	/// Funktion berechnet den Preis der Ressourcen in Zusammenhang zu den anderen Brsen. 
	/// @param pmMajors Zeiger auf Map mit allen Majors des Spiels
	/// @param pCurMajor aktuelle Rasse, fr die die Preise berechnet werden
	void CalculatePrices(map<CString, CMajor*>* pmMajors, CMajor* pCurMajor);

	// Funktion veranlat, dass in der neuen Runde versucht wird ein Monopol zu erlangen.
	void BuyMonopol(USHORT res, double monopol_costs) {m_dMonopolBuy[res] = monopol_costs;}

	// Resetfunktion fr die Klasse CTrade
	void Reset(void);
	
private:

	// Das Handelshistoryobjekt, welches alle vergangenen Brsenkurse gespeichert hat
	CTradeHistory m_TradeHistory;	

	// Der aktuelle Preis der jeweiligen Ressource an der globalen Handelsbrse
	USHORT m_iRessourcePrice[IRIDIUM + 1];

	// Der Preis der Ressource zu Beginn einer Runde (wird genommen, wenn wir Bauauftrge kaufen, sonst knnte man
	// in der gleichen Runde den Preis drcken, dann billig kaufen und dann den Preis wieder hochtreiben)
	USHORT m_iRessourcePriceAtRoundStart[IRIDIUM + 1];
	
	// Die Anzahl der jeweiligen Ressource die wir kaufen oder verkaufen mchten (negative Werte bedeuten verkaufen)
	CArray<m_TradeStruct,m_TradeStruct> m_TradeActions;	
	
	// Die Menge die wir bei einem Klick kaufen bzw. Verkaufen
	USHORT m_iQuantity;

	// Variable die die aktuelle Steuer auf Handelsaktivitten festhlt (rassenabhngig)
	float m_fTax;

	// Steuergelder auf Ressourcen nur durch Sofortkauf von Bauauftrgen, nicht die Steuern, die wir durch normalen
	// Handel machen
	USHORT m_iTaxes[IRIDIUM + 1];
	
	// Welche Majorrace besitzt ein Monopol auf die jeweilige Ressource
	static CString m_sMonopolOwner[IRIDIUM + 1];

	// Wollen wir ein Monopol kaufen? Wird bei neuer Runde abgefragt. Ist der Wert darin ungleich NULL, dann wollen wir
	// eins kaufen. Der Wert gibt auch den Kaufpreis an
	double m_dMonopolBuy[IRIDIUM + 1];
};

