/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once
#include "troop.h"

class CTroopInfo :	public CTroop
{
public:
	/// Konstruktor
	CTroopInfo(void);

	/// Konstruktor mit kompletter Parameterbergabe
	CTroopInfo(const CString& name, const CString& desc,const CString& file, BYTE offense, BYTE defense, BYTE costs, BYTE techs[6], USHORT res[5], USHORT ip,
		BYTE ID, const CString& sOwner, USHORT size, BYTE moralValue);
	
	/// Destruktor
	~CTroopInfo(void);
	
	// Serialisierungsfunktion; ntig fr CBotf2Doc::SerializeBeginGameData
	virtual void Serialize(CArchive &ar);

	// Zugriffsfunktionen zum Lesen der Membervariablen
	/**
	 * Diese Funktion gibt den Namen der Truppe zurck.
	 */
	const CString& GetName() const {return m_strName;}

	/**
	 * Dieser Funktion gibt die Beschreibung der Truppe zurck.
	 */
	const CString& GetDescription() const {return m_strDescription;}
	
	/*
	 *Diese Funktion gibt das Bild der Truppe zurck
	 */
	const CString& GetGraphicfile() const {return m_strGraphicfile;}

	/**
	 * Diese Funktion gibt die Unterhaltskosten der Truppe zurck.
	 */
	BYTE GetMaintenanceCosts() const {return m_byMaintenanceCosts;}

	/**
	 * Diese Funktion gibt einen Zeiger auf die bentigten Techlevel dieser Einheit zurck.
	 */
	BYTE* GetNeededTechlevels() {return m_byNeededTechs;}

	/**
	 * Diese Funktion gibt ein bentigtes Techlevel zurck, welches man mit dem Paramter <code>tech</code>
	 * erfragt hat.
	 */
	BYTE GetNeededTechlevel(BYTE tech) const {return m_byNeededTechs[tech];}

	/**
	 * Diese Funktion gibt einen Zeiger auf die bentigten Ressourcen dieser Truppe zurck
	 */
	USHORT* GetNeededResources() {return m_iNeededResources;}

	/**
	 * Diese Funktion gibt die bentigte Industrie zum Bau dieser Einheit zurck.
	 */
	USHORT GetNeededIndustry() const {return m_iNeededIndustry;}

	/**
	 * Diese Funktion gibt die Truppengre, somit den bentigten Platz bei einem Transport zurck.
	 */
	USHORT GetSize() const {return m_iSize;}

	/**
	 * Diese Funktion gibt den Moralwert dieser Truppe zurck.
	 */
	BYTE GetMoralValue() const {return m_byMoralValue;}
	
	// Zugriffsfunktionen zum Schreiben der Membervariablen
	/**
	 * Diese Funktion setzt den Namen der Einheit.
	 */
	void SetName(const CString& name) {m_strName = name;}

	/**
	 * Diese Funktion setzt die Beschreibung der Truppe.
	 */
	void SetDescription(const CString& desc) {m_strDescription = desc;}


	/**
	 *Diese Funktion setzt das Bild der Truppe
	 */
	void SetGraphicfile(const CString& name) {m_strGraphicfile = name;}

	/**
	 * Diese Funktion setzt die zum Bau bentigte Industrie der Truppe.
	 */
	void SetNeededIndustry(USHORT ip) {m_iNeededIndustry = ip;}

	/// Funktion gibt zurck, ob die Truppe mit der aktuellen Forschung einer Rasse baubar ist.
	/// @param researchLevels Forschungsstufen der Rasse
	/// @return Wahrheitswert
	bool IsThisTroopBuildableNow(const BYTE reserachLevels[6]) const;

private:
	/// Der Name der Einheit
	CString m_strName;

	/// Die Beschreibung der Einheit
	CString m_strDescription;
	
	/// Das Bild der Einheit
	CString m_strGraphicfile;

	/// Diese Variable beinhaltet die Unterhaltskosten der Einheit
	BYTE m_byMaintenanceCosts;

	/// Dieses Feld beinhaltet die bentigten Forschungslevel zum Bau dieser Einheit
	BYTE m_byNeededTechs[6];

	/// Dieses Feld beinhaltet die bentigten Ressourcen zum Bau dieser Einheit
	USHORT m_iNeededResources[5];

	/// Die bentigte Industrie um eine Einheit fertigzustellen
	USHORT m_iNeededIndustry;

	/// Die Anzahl der Soldaten bzw. der bentigte Platz bei einem Transport fr diese Einheit
	USHORT m_iSize;

	/// Der Moralwert der Truppe. Um dieses Wert wird die Moral bei unter 100% im System erhht bzw.
	/// bei ber 100% in dem System verringert
	BYTE m_byMoralValue;
};
