/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once

/// Singletonklasse um Daten aus der Ini-Datei zu lesen bzw. zu schreiben.
class CIniLoader
{
private:	
	/// Konstruktor private, damit man sich keine Instanzen holen kann.
    CIniLoader(void);
    
	/// Den Kopierkonstruktor schtzen um zu vermeiden, dass das Objekt unbeabsichtigt kopiert wird.
    CIniLoader(const CIniLoader& cc);

public:
	/// Standarddestruktor
	~CIniLoader(void);

	/// Funktion liefert die einzige Instanz dieser Klasse (Singleton).
	/// @return Instanz dieser Klasse
	static CIniLoader* GetInstance(void);

	/**
	 * Diese Funktion gibt den zum <code>key</code> gehrenden Eintrag aus der ini-Datei zurck.
	 */
	bool ReadValue(const CString& sSection, const CString& sKey, bool& bValue) const;

	/**
	 * Diese Funktion gibt den zum <code>key</code> gehrenden Eintrag aus der ini-Datei zurck.
	 */
	bool ReadValue(const CString& sSection, const CString& sKey, int& nValue) const;

	/**
	 * Diese Funktion gibt den zum <code>key</code> gehrenden Eintrag aus der ini-Datei zurck.
	 */
	bool ReadValue(const CString& sSection, const CString& sKey, float& fValue) const;

	/**
	 * Diese Funktion gibt den zum <code>key</code> gehrenden Eintrag aus der ini-Datei zurck.
	 */
	bool ReadValue(const CString& sSection, const CString& sKey, CString& sValue) const;

	/**
	 * Diese Funktion schreibt den zum <code>key</code> gehrenden Eintrag <code>value</code> in die ini-Datei.
	 */
	bool WriteValue(const CString& sSection, const CString& sKey, const CString& sValue);

private:
	/**
	 * Diese Funktion wandelt ein Wort, wie z.B. On oder True in einen Wahrheitswert um.
	 */
	int StringToInt(const CString& sValue) const;	

	/**
	 * Diese Funktion gibt den zum <code>key</code> gehrenden Eintrag aus der ini-Datei zurck.
	 */
	bool ReadIniValue(const CString& sSection, const CString& sKey, CString& sReturnValue) const;

	// Attribute
	CString m_sIniPath;		///< kompletter Pfad zur Ini-Datei
};
