/*
 *   Copyright (C)2004-2011 Sir Pustekuchen
 *
 *   Author   :  Sir Pustekuchen
 *   Home     :  http://birth-of-the-empires.de
 *
 */
#pragma once

#include "fmod.hpp"
#include "network.h"

#include <list>
#include <string>

/**
 * Konstanten fr Hintergrundmusik und Gerusche, die die Oberflche erzeugen kann.
 */
typedef enum SNDMGR_VALUE
{
	SNDMGR_INVALID,								///< ungltiger Wert

	SNDMGR_MUSIC_FED,							///< Hintergrundmusik Konfderation
	SNDMGR_MUSIC_FER,							///< Hintergrundmusik Hanuhr
	SNDMGR_MUSIC_KLI,							///< Hintergrundmusik Khayrin
	SNDMGR_MUSIC_ROM,							///< Hintergrundmusik Rotharianer
	SNDMGR_MUSIC_CAR,							///< Hintergrundmusik Cartarer
	SNDMGR_MUSIC_DOM,							///< Hintergrundmusik Omegas

	SNDMGR_VOICE_FED_RACESELECT,				///< Stimme zur Rassenauswahl fr Konfderation
	SNDMGR_VOICE_FER_RACESELECT,				///< Stimme zur Rassenauswahl fr Hanuhr
	SNDMGR_VOICE_KLI_RACESELECT,				///< Stimme zur Rassenauswahl fr Khayrin
	SNDMGR_VOICE_ROM_RACESELECT,				///< Stimme zur Rassenauswahl fr Rotharianer
	SNDMGR_VOICE_CAR_RACESELECT,				///< Stimme zur Rassenauswahl fr Cartarer
	SNDMGR_VOICE_DOM_RACESELECT,				///< Stimme zur Rassenauswahl fr Omegas

	SNDMGR_SOUND_MAINMENU,						///< Button im Hauptmen
	SNDMGR_SOUND_ENDOFROUND,					///< Rundenende-Button
	SNDMGR_SOUND_SHIPTARGET,					///< Ziel fr Schiff auswhlen
	SNDMGR_SOUND_ERROR,							///< Fehlerpiepsen

	SNDMGR_MSG_BATTLELOST,
	
	SNDMGR_MSG_ALIENCONTACT,					///< Alienkontakt wurde hergestellt (Minorrace getroffen)
	SNDMGR_MSG_CLAIMSYSTEM,						///< neue Kolonie / neues System
	SNDMGR_MSG_COLONIZING,						///< Kolonisierung gestartet
	SNDMGR_MSG_DIPLOMATICNEWS,					///< neue diplomatische Nachrichten eingetroffen
	SNDMGR_MSG_FIRSTCONTACT,					///< Erstkontakt mit einer Majorrace
	SNDMGR_MSG_INTELNEWS,						///< neue Geheimdienstnachrichten sind eingetroffen
	SNDMGR_MSG_NEWTECHNOLOGY,					///< neue Technologie erforscht
	SNDMGR_MSG_OUTPOST_CONSTRUCT,				///< ein Aussenposten wird gebaut
	SNDMGR_MSG_OUTPOST_READY,					///< Bau eines Aussenpostens abgeschlossen
	SNDMGR_MSG_SCIENTISTNEWS,					///< technologischer Durchbruch
	SNDMGR_MSG_STARBASE_CONSTRUCT,				///< eine Sternbasis wird gebaut
	SNDMGR_MSG_STARBASE_READY,					///< Bau einer Sternbasis abgeschlossen
	SNDMGR_MSG_TERRAFORM_COMPLETE,				///< Terraforming ist fertig
	SNDMGR_MSG_TERRAFORM_SELECT,				///< Planet zum Terraformen auswhlen
	
	// TODO ergnzen

}
SNDMGR_VALUE;

/**
 * Prioritten fr Gerusche.
 */
typedef enum
{
	SNDMGR_PRIO_NORMAL,							///< normale Prioritt
	SNDMGR_PRIO_HIGH,							///< hohe Prioritt, aber niedriger als die Hintergrundmusik
}
SNDMGR_PRIO;

/**
 * Typ eines Eintrags in <code>SNDMGR_SOUNDLIST</code>.
 */
typedef struct SNDMGR_SOUNDENTRY
{
	SNDMGR_VALUE nType;							///< <code>SNGMGR_SOUND</code>-Konstante
	FMOD::Sound *pSound;						///< geladenes <code>FMOD::Sound</code>-Objekt
}
SNDMGR_SOUNDENTRY;

/**
 * Typ fr <code>CSoundManager::m_lSounds</code>.
 */
typedef std::list<SNDMGR_SOUNDENTRY> SNDMGR_SOUNDLIST;

/**
 * Eintrag in <code>SNDMGR_MESSAGELIST</code>.
 */
typedef struct SNDMGR_MESSAGEENTRY
{
	SNDMGR_VALUE nMessage;						///< Typ der Nachricht
	network::RACE nRace;						///< "Stimme", mit der die Nachricht abgespielt wird
	int nPriority;								///< Prioritt (0 Standard/geringste)
	float fVolume;								///< Lautstrke, mit der abgespielt werden soll
}
SNDMGR_MESSAGEENTRY;

/**
 * Typ fr <code>CSoundManager::m_lMessages</code>.
 */
typedef std::list<SNDMGR_MESSAGEENTRY> SNDMGR_MESSAGELIST;

/**
 * Verwaltet das Abspielen von Hintergrundmusik und Vordergrundgeruschen.
 *
 * @author CBot
 * @version 0.0.2
 */
class CSoundManager
{
private:

	static const int PRIORITY_SOUND_HIGH = 127;		///< hohe Prioritt der Gerusche, die die Oberflche erzeugt
	static const int PRIORITY_SOUND_NORMAL = 128;	///< normale Prioritt der Gerusche, die die Oberflche erzeugt

	/**
	 * Konstruktor private, damit man sich keine Instanzen holen kann.
	 * @param nMaxLoadedSound maximale Anzahl gleichzeitig geladener Sounds
	 */
	CSoundManager(UINT nMaxLoadedSounds = 10);

	/// Den Kopierkonstruktor schtzen um zu vermeiden, dass das Objekt unbeabsichtigt kopiert wird.
    CSoundManager(const CSoundManager& cc);

	FMOD_RESULT m_nLastResult;						///< letzter Rckgabewert einer FMOD-Funktion
	FMOD::System *m_pSystem;

	FMOD::Channel *m_pMusicChannel;					///< Kanal, in dem die Hintergrundmusik abgespielt wird
	FMOD::Sound *m_pMusic;							///< Hintergrundmusik

	/// Liste der geladenen Sounds; das zuletzt verwendete Objekt wird ans Ende der Liste gesetzt; der erste
	/// Eintrag entspricht demjenigen Sound, dessen letzte Verwendung am lngsten her ist
	SNDMGR_SOUNDLIST m_lSounds;
	UINT m_nMaxLoadedSounds;						///< maximale Anzahl gleichzeitig geladener Sounds
	FMOD::Channel *m_pSoundChannel;					///< Kanal, in dem die Sounds abgespielt werden
	float m_fSoundMasterVolume;						///< Master-Lautstrke fr Sounds

	SNDMGR_MESSAGELIST m_lMessages;					///< Liste der noch abzuspielenden Nachrichten
	FMOD::Channel *m_pMessageChannel;				///< Kanal, in dem Nachrichten abgespielt werden
	FMOD::Sound *m_pLastMessage;					///< aktuell abzuspielende Nachricht
	bool m_bMessagesPlaying;						///< <code>true</code> gdw. Messages gerade abgespielt werden
	bool m_bInterruptMessages;						///< soll Abspielen der Nachrichten unterbrochen werden?
	DWORD m_dwFirstMillis;							///< Pause vor dem Abspielen der ersten Nachricht (Millisekunden)
	DWORD m_dwMillis;								///< Zeit zwischen zwei Nachrichten (Millisekunden)
	HANDLE m_hThreadStopped;						///< Event, das ausgelst wird, wenn der Abspielthread beendet ist
	float m_fMessageMasterVolume;					///< Master-Lautstrke fr Nachrichten

	BOOL m_bUseSoftwareSound;						///< Soundausgabe hardwarebeschleunigt oder per Software

public:
	/**
	 * Hlt den Thread an, gibt smtliche Ressourcen frei.
	 */
	virtual ~CSoundManager();

	/// Funktion liefert die einzige Instanz dieser Klasse (Singleton).
	/// @return Instanz dieser Klasse
	static CSoundManager* GetInstance(void);

	/**
	 * Setzt die maximale Anzahl gleichzeitig zu ladender Oberflchen-Sounds. Wird dieser Wert auf einen
	 * kleineren als bisher gesetzt, wird die entsprechende Anzahl geladener Sounds beim nchsten Aufruf
	 * von <code>PlaySound()</code> freigegeben.
	 */
	void SetMaxLoadedSounds(UINT nMaxLoadedSounds) {m_nMaxLoadedSounds = nMaxLoadedSounds;}
	UINT GetMaxLoadedSounds() const {return m_nMaxLoadedSounds;}

	/// @return Anzahl der gleichzeitig geladenen Sounds
	UINT GetLoadedSounds() const {return (UINT)m_lSounds.size();}

	/**
	 * Gibt alle geladenen Oberflchen-Sounds frei. Wird einer der Sounds abgespielt, so wird das
	 * Abspielen durch das Freigeben unterbrochen.
	 */
	void UnloadAllSounds();

	/**
	 * Liefert den zuletzt aufgetretenen Fehler, wenn eine Methode von <code>CSoundManager</code>
	 * den Wert <code>FALSE</code> bwz. <code>NULL</code> geliefert hat. Sonst ist der gelieferte Wert
	 * undefiniert.
	 */
	FMOD_RESULT GetLastError() const {return m_nLastResult;}

	/**
	 * Initialisiert das Sound-System.
	 * @return <code>TRUE</code> genau bei Erfolg
	 */
	BOOL Init(BOOL bUseSoftwareSound = FALSE);

	/**
	 * Gibt das Sound-System und alle Ressourcen wieder frei.
	 *
	 * Das Sound-System kann mit <code>Init()</code> anschlieend wieder initialisiert werden.
	 */
	void Release();

	/**
	 * Startet das Abspielen der angegebenen Datei als Hintergrundmusik. Bisher abgespielte Hintergrundmusik
	 * wird unterbrochen und freigegeben.
	 *
	 * @return <code>TRUE</code> genau bei Erfolg
	 */
	BOOL StartMusic(network::RACE race, float fVolume = 1.0f);

	/**
	 * Startet das Abspielen der angegebenen Datei als Hintergrundmusik. Bisher abgespielte Hintergrundmusik
	 * wird unterbrochen und freigegeben.
	 *
	 * @return <code>TRUE</code> genau bei Erfolg
	 */
	BOOL StartMusic(const CString& sFile, float fVolume = 1.0f);

	/**
	 * Hlt die Hintergrundmusik an und gibt sie frei.
	 */
	void StopMusic();

	/**
	 * Blendet die abgespielte Hintergrundmusik aus. Dafr wird die Lautstrke in <code>nSteps</code> Schritten
	 * jeweils nach <code>dwMilliseconds</code> Millisekunden um <code>fVolume / nSteps</code> gesenkt,
	 * wobei <code>fVolume</code> die gesetzte Lautstrke der Hintergrundmusik beim Aufruf dieser Methode ist. Bei
	 * <code>nSteps == 0</code> wird die Lautstrke ohne zu Warten sofort auf <code>0</code> gesetzt.
	 *
	 * Diese Methode kehrt erst zurck, wenn die Hintergrundmusik die Lautstrke <code>0</code> erreicht hat.
	 */
	void FadeOutMusic(UINT nSteps = 20, DWORD dwMilliseconds = 100);

	/**
	 * Setzt die Lautstrke der Hintergrundmusik, wenn diese luft.
	 */
	void SetMusicVolume(float fVolume);

	/**
	 * Hlt die Hintergrundmusik an bzw. setzt sie fort.
	 */
	void PauseMusic(BOOL bPaused = TRUE);

	/**
	 * Spielt den angegebenen Oberflchen-Sound ab. Wird bereits ein Sound abgespielt, wird dieses unterbrochen.
	 * @return <code>TRUE</code> genau bei Erfolg
	 */
	BOOL PlaySound(SNDMGR_VALUE nSound, SNDMGR_PRIO nPriority = SNDMGR_PRIO_NORMAL, float fVolume = 1.0f, network::RACE nRace = network::RACE_NONE);

	/**
	 * Setzt die Master-Lautstrke fr Oberflchen-Sounds. Ein Wert von <code>0</code> schaltet die Sound-
	 * Ausgabe ab.
	 */
	void SetSoundMasterVolume(float fSoundMasterVolume);

	/**
	 * Hngt an die Liste der abzuspielenden Nachrichten die gegebene Nachricht an. Diese wird spter mit der
	 * "Stimme" des angegebenen Volkes abgespielt. Es kann eine Priotitt angegeben werden. Nachrichten mit
	 * greren Werten von <code>nPriority</code> werden eher abgespielt.
	 */
	void AddMessage(SNDMGR_VALUE nMsg, network::RACE nRace = network::RACE_NONE, int nPriority = 0,
		float fVolume = 1.0f);

	/**
	 * Leert die Liste der Nachrichten.
	 */
	void ClearMessages();

	/**
	 * Setzt die Master-Lautstrke fr Nachrichten. Ein Wert von <code>0</code> schaltet die Wiedergabe ab.
	 */
	void SetMessageMasterVolume(float fMessageMasterVolume);

	/**
	 * Sortiert die Liste der Nachrichten nach Prioritt und spielt diese der Reihe nach ab. Vor der ersten
	 * Nachricht wird <code>dwFirstMillis</code> Millisekunden gewartet, zwischen zwei Nachrichten wird
	 * <code>dwMillis</code> Millisekunden pausiert.
	 */
	BOOL PlayMessages(DWORD dwFirstMillis = 0, DWORD dwMillis = 0);

	/**
	 * Unterbricht das Abspielen der Nachrichten. Fr <code>bHardInterrupt = FALSE</code> wird eine begonnene
	 * Nachricht bis zum Ende abgespielt, sonst wird diese sofort unterbrochen. Ein Aufruf von
	 * <code>PlayMessages()</code> startet das Abspielen wieder und setzt mit der nchsten Nachricht fort.
	 *
	 * Im Fall des sofortigen Unterbrechens kehrt die Methode erst zurck, wenn der Abspielthread beendet wurde
	 * oder der angegebene Timeout abgelaufen ist.
	 *
	 * @return beim normalen Unterbrechen immer <code>TRUE</code>; beim sofortigen Unterbrechen <code>TRUE</code>,
	 * wenn der Thread beendet wurde, bevor der Timeout abgelaufen ist
	 */
	BOOL StopMessages(BOOL bHardInterrupt = FALSE, DWORD dwTimeout = INFINITE);

private:
	/**
	 * Liefert den Zeiger auf das zum angegebenen Sound gehrige <code>FMOD::Sound</code>-Objekt. Es werden maximal
	 * <code>CSoundManager::m_nMaxLoadedSounds</code> verschiedene Sounds gleichzeitig geladen. Muss bei maximaler
	 * Auslastung ein weiterer Sound geladen werden, wird der Sound, dessen letztes Abspielen am lngsten zurck liegt,
	 * entfernt.
	 * @return geladenes Sound-Objekt bei Erfolg; sonst <code>NULL</code>, <code>CSoundManager::GetLastError()</code>
	 * liefert dann den Fehler
	 */
	FMOD::Sound *LoadSound(SNDMGR_VALUE nSound, network::RACE nRace = network::RACE_NONE);

	/**
	 * @return den zum angegebenen Sound gehrigen Dateinamen
	 */
	static std::string GetPathName(SNDMGR_VALUE nSound, network::RACE nRace = network::RACE_NONE);

	/**
	 * @return die dem angegebenen Volk zugeordnete Hintergrundmusik als
	 * <code>SNDMGR_VALUE</code>-Konstante; <code>SNDMGR_INVALID</code> wenn Volk ungltig oder keine
	 * Zuordnung angegeben ist
	 */
	static SNDMGR_VALUE GetRaceMusic(network::RACE race);

	static UINT ThreadProc(LPVOID pParam);

	/**
	 * Entfernt die nchste Nachricht aus der Liste und spielt diese ab.
	 */
	BOOL PlayNextMessage();

	static bool CompareMessageEntries(const SNDMGR_MESSAGEENTRY &entry1, const SNDMGR_MESSAGEENTRY &entry2);

};
