#pragma once

#include "afxmt.h"

/**
 * Basisklasse fr Klassen, deren Objekte mit Hilfe von <code>CReference</code> und
 * <code>CReferenceTransfer</code> referenzgezhlt werden sollen.
 *
 * @author CBot
 * @version 0.0.1
 */
class CReferenceCounter
{
private:
	UINT m_nCounter;							///< Referenzzhler
	CCriticalSection m_CriticalSection;

public:
	/**
	 * Erzeugt ein Objekt. Initialisiert den Referenzzhler mit <code>0</code>. Anschlieend
	 * wird mit der Konstruktion von <code>CReference</code> oder <code>CReferenceTransfer</code>
	 * <code>Pin()</code> aufgerufen, so dass ein spterer Aufruf von <code>Release()</code>
	 * dieses Objekt zerstrt.
	 */
	CReferenceCounter();
	virtual ~CReferenceCounter();

	/**
	 * @return aktueller Referenzzhler, nach der Konstruktion <code>0</code>, nach dem ersten
	 * Aufruf von <code>Pin()</code> immer <code>&gt;= 1</code>
	 */
	inline UINT GetReferenceCount() const {return m_nCounter;}

	/**
	 * Erhht den Referenzzhler. Diese Methode ist synchronisiert.
	 */
	void Pin();
	/**
	 * Dekrementiert den Referenzzhler. Zerstrt dieses Objekt, wenn der Zhler mit dem Aufruf
	 * dieser Methode auf <code>0</code> fllt. Diese Methode ist synchronisiert.
	 */
	void Release();

};
