#pragma once

template <class T> class CReferenceTransfer;

#include "Reference.h"

/**
 * Reprsentiert eine Referenz auf ein Objekt vom Typ <code>T</code>. Dient speziell der bergabe
 * des Besitzes von Objekten, ohne den Referenzzhler zu in- und sofort wieder zu dekrementieren.
 *
 * @author CBot
 * @version 0.0.1
 */
template <class T> class CReferenceTransfer
{
	friend class CReference<T>;

private:
	T *m_pObject;					///< Zeiger auf das referenzierte Objekt oder <code>NULL</code>

public:
	/**
	 * Erzeugt eine neue Referenz auf das angegebene Objekt; inkrementiert dessen Referenzzhler.
	 */
	CReferenceTransfer(T *pObject = NULL) : m_pObject(pObject)
	{
		if (pObject) pObject->Pin();
	}

	/**
	 * bernimmt die Referenz von <code>other</code>. <code>other</code> referenziert anschlieend nichts mehr.
	 * Der Referenzzhler des referenzierten Objektes bleibt unverndert.
	 */
	CReferenceTransfer(CReferenceTransfer<T> &other)
	{
		m_pObject = other.Detach();
	}

	/**
	 * bernimmt die Referenz von <code>other</code>. <code>other</code> referenziert anschlieend nichts mehr.
	 * Der Referenzzhler des referenzierten Objektes bleibt unverndert.
	 */
	CReferenceTransfer(CReference<T> &other)
	{
		m_pObject = other.Detach();
	}

	/**
	 * Zerstrt diese Referenz; dekrementiert den Zhler des referenzierten Objekts.
	 */
	virtual ~CReferenceTransfer()
	{
		if (m_pObject) m_pObject->Release();
	}

	/**
	 * Gibt die Referenz auf das zuvor referenzierte Objekt frei, erzeugt eine Referenz auf das zugewiesene
	 * Objekt.
	 */
	inline void operator=(T *pObject)
	{
		if (m_pObject) m_pObject->Release();
		m_pObject = pObject;
		if (m_pObject) m_pObject->Pin();
	}

	/**
	 * Trennt das referenzierte Objekt von dieser Referenz, ohne dessen Referenzzhler zu dekrementieren.
	 * Diese Referenz referenziert anschlieend kein Objekt mehr.
	 *
	 * @return Zeiger auf das bisher referenzierte Objekt
	 */
	T *Detach()
	{
		T *result = m_pObject;
		m_pObject = NULL;
		return result;
	}

};
