#pragma once

#include "Message.h"

#pragma warning (disable: 4244)

namespace network
{
	typedef CList<RACE> RACE_LIST;

	/**
	 * Eine Chat-Nachricht. Wird von einem Client an den Server gesendet, der diese an die
	 * adressierten Empfnger verteilt.
	 *
	 * Aufbau:
	 * <pre>
	 * CString	senderName		Name des Absenders; wird vom Server ggf. korrigiert
	 * BYTE		sender			Absender der Nachricht (RACE), whrend der Startphase immer RACE_NONE; wird vom Server
	 *							ggf. korrigiert
	 * BYTE		recvCount		Anzahl der Empfnger, > 0
	 * BYTE[]	receivers		Empfnger der Nachricht (RACE), recvCount Eintrge; ist einer der Eintrge
	 *							RACE_ALL, wird die Nachricht an alle verbundenen Clients gesendet (whrend
	 *							der Startphase also auch an Clients, denen kein Volk zugeordnet ist)
	 * CString	msg				Nachricht
	 * </pre>
	 *
	 * @author CBot
	 * @version 0.0.2
	 */
	class CChatMsg : public CMessage
	{
	private:
		CString m_sSenderName;			///< Name des Absenders
		/// Volk des Absenders; beim Erzeugen immer <code>RACE_NONE</code>, wird vom Server korrekt ausgefllt
		RACE m_nSender;
		/// Liste der Empfnger der Nachricht; enthlt entweder nur Eintrge <code>!= RACE_ALL</code> oder genau
		/// einen Eintrag mit dem Wert <code>RACE_ALL</code>
		RACE_LIST m_lReceivers;
		CString m_sMsg;					///< Nachricht

	public:
		CChatMsg();

		/// Erzeugt ein Chat-Nachrichtenobjekt. Whitespaces am Beginn und Ende der Nachricht werden abgeschnitten.
		CChatMsg(CString msg);
		/// Erzeugt ein Chat-Nachrichtenobjekt. Whitespaces am Beginn und Ende des Namens und der Nachricht
		/// werden abgeschnitten. Es wird der angegebene Empfnger zur Liste der Empfnger hinzugefgt.
		CChatMsg(CString msg, RACE receiver);
		virtual ~CChatMsg();

		/// Fgt einen Empfnger zur Liste der Empfnger hinzu. Die Angabe von <code>RACE_ALL</code> ist mglich,
		/// wobei alle anderen Eintrge entfernt werden.
		void AddReceiver(RACE race);
		/// Setzt das Volk des Absenders dieser Chatnachricht. Sendet der Client eine Chatnachricht an den Server,
		/// wird der Wert des Absendervolkes vom Server durch den tatschlichen Wert ersetzt. Whrend der
		/// Zuordnungsphase <code>RACE_NONE</code>.
		void SetSender(RACE nSender);
		/// Setzt den Benutzernamen des Absenders. Wird vom Server durch den Namen ersetzt, der beim Herstellen
		/// der Verbindung angegeben wurde.
		void SetSenderName(CString strSender);

		CString GetSenderName() const {return m_sSenderName;}
		RACE GetSenderRace() const {return m_nSender;}
		CString GetMessage() const {return m_sMsg;}
		UINT GetReceiverCount() const {return m_lReceivers.GetCount();}
		/// @return einen Empfnger; <code>RACE_NONE</code>, wenn ein ungltiger Index angegeben wurde
		RACE GetReceiver(int idx) const;

		BOOL IsForAllClients() const;

		virtual void Serialize(CArchive &ar);

	};

};
